@@ -117,7 +117,7 @@ int SynthCallback(short *wav, int numsamples, espeak_EVENT *events)
 			Event = &Events[n_Events++];
 			Event->eEventId             = SPEI_TTS_BOOKMARK;
 			Event->elParamType          = SPET_LPARAM_IS_STRING;
-			Event->ullAudioStreamOffset = ((event->audio_position + audio_offset) * 441)/10;  // ms -> bytes
+			Event->ullAudioStreamOffset = ((event->audio_position + audio_offset) * srate)/10;  // ms -> bytes
 			Event->lParam               = (long)event->id.name;
 			Event->wParam               = wcstol((wchar_t *)event->id.name,&tailptr,10);
 		}
@@ -125,11 +125,11 @@ int SynthCallback(short *wav, int numsamples, espeak_EVENT *events)
 		if(event->type == espeakEVENT_SENTENCE)
 		{
 			Event = &Events[n_Events++];
-            Event->eEventId             = SPEI_SENTENCE_BOUNDARY;
-            Event->elParamType          = SPET_LPARAM_IS_UNDEFINED;
-            Event->ullAudioStreamOffset = (event->audio_position * 441)/10;  // ms -> bytes
-            Event->lParam               = event->text_position-1 + text_offset;
-            Event->wParam               = 0;  // TEMP
+			Event->eEventId             = SPEI_SENTENCE_BOUNDARY;
+			Event->elParamType          = SPET_LPARAM_IS_UNDEFINED;
+			Event->ullAudioStreamOffset = ((event->audio_position + audio_offset) * srate)/10;  // ms -> bytes
+			Event->lParam               = 0;
+			Event->wParam               = 0;  // TEMP
 		}
 #endif
 	}
@@ -148,8 +148,8 @@ static int ConvertRate(int new_rate)
 
 	int rate;
 
-	static int rate_table[21] = {80,100,116,124,132,140,148,156,164,170,176,
-				182,188,197,208,220,240,270,300,335,370 };
+	static int rate_table[21] = {80,100,115,124,133,142,151,159,168,174,180,
+				187,196,208,220,240,270,300,335,369,390 };
 
 	rate = new_rate + master_rate;
 	if(rate < -10) rate = -10;
@@ -298,10 +298,100 @@ STDMETHODIMP CTTSEngObj::SetObjectToken(ISpObjectToken * pToken)
 //=== ISpTTSEngine Implementation ============================================
 //
 
+#define L(c1,c2)  (c1<<8)+c2          // combine two characters into an integer
+
+static char *phoneme_names_en[] = {
+	NULL,NULL,NULL," ",NULL,NULL,NULL,NULL,"'",",",
+	"A:","a","V","0","aU","@","aI",
+	"b","tS","d","D","E","3:","eI",
+	"f","g","h","I","i:","dZ","k",
+	"l","m","n","N","oU","OI","p",
+	"r","s","S","t","T","U","u:",
+	"v","w","j","z","Z",
+	NULL
+ };
+
+
+
+int CTTSEngObj::WritePhonemes(SPPHONEID *phons, wchar_t *pW)
+{//=========================================================
+	int ph;
+	int ix=2;
+	int skip=0;
+	int maxph = 49;
+	char *p;
+	int j;
+	int lang;
+	char **phoneme_names;
+	char phbuf[200];
+	espeak_VOICE *voice;
+
+	voice = espeak_GetCurrentVoice();
+	lang = (voice->languages[1] << 8) + (voice->languages[2]);
+
+	phoneme_names = phoneme_names_en;
+	maxph = 0;
+
+	if(lang == L('e','n'))
+	{
+		phoneme_names = phoneme_names_en;
+		maxph = 49;
+	}
+
+	if(maxph == 0)
+		return(0);
+
+	strcpy(phbuf,"[[");
+	while(((ph = *phons++) != 0) && (ix < (sizeof(phbuf) - 3)))
+	{
+		if(skip)
+		{
+			skip = 0;
+			continue;
+		}
+		if(ph > maxph)
+			continue;
+
+		p = phoneme_names[phons[0]];  // look at the phoneme after this one
+		if(p != NULL)
+		{
+			if(p[0] == '\'')
+			{
+				phbuf[ix++] = '\'';  // primary stress, put before the vowel, not after
+				skip=1;
+			}
+			if(p[0] == ',')
+			{
+				phbuf[ix++] = ',';  // secondary stress
+				skip=1;
+			}
+		}
+
+		p = phoneme_names[ph];  // look at this phoneme
+
+		if(p != NULL)
+		{
+			strcpy(&phbuf[ix],p);
+			ix += strlen(p);
+		}
+	}
+	strcpy(&phbuf[ix],"]]");
+	ix += 2;
+
+	if(pW != NULL)
+	{
+		for(j=0; j<=ix; j++)
+		{
+			pW[j] = phbuf[j];
+		}
+	}
+	return(strlen(phbuf));
+}
+
 
 
 int CTTSEngObj::ProcessFragList(const SPVTEXTFRAG* pTextFragList, wchar_t *pW_start, ISpTTSEngineSite* pOutputSite, int *n_text)
-{//=============================================================================================================================
+{//============================================================================================================================
 
 	int action;
 	int control;
@@ -338,6 +428,7 @@ int CTTSEngObj::ProcessFragList(const SPVTEXTFRAG* pTextFragList, wchar_t *pW_st
 
 		CheckActions(pOutputSite);
 		sayas = 0;
+		state = &pTextFragList->State;
 
 		switch(action)
 		{
@@ -366,7 +457,6 @@ int CTTSEngObj::ProcessFragList(const SPVTEXTFRAG* pTextFragList, wchar_t *pW_st
 			}
 
 			// first set the volume, rate, pitch
-			state = &pTextFragList->State;
 			volume = (state->Volume * master_volume)/100;
 			speed = ConvertRate(state->RateAdj);
 			pitch = ConvertPitch(state->PitchAdj.MiddleAdj);
@@ -467,6 +557,14 @@ int CTTSEngObj::ProcessFragList(const SPVTEXTFRAG* pTextFragList, wchar_t *pW_st
 				}
 			}
 			break;
+
+		case SPVA_Pronounce:
+			total += WritePhonemes(state->pPhoneIds, pW);
+			if(pW != NULL)
+			{
+				pW += total;
+			}
+			break;
 		}
 
 
@@ -478,6 +576,7 @@ int CTTSEngObj::ProcessFragList(const SPVTEXTFRAG* pTextFragList, wchar_t *pW_st
 		*pW = 0;
 	}
 	*n_text = frag_count;
+
 	return(total);
 }   // end of ProcessFragList
 
@@ -600,7 +699,7 @@ STDMETHODIMP CTTSEngObj::Speak( DWORD dwSpeakFlags,
 
 		if(size > 0)
 		{
-			espeak_Synth(TextBuf,0,0,POS_CHARACTER,0,espeakCHARS_WCHAR | espeakKEEP_NAMEDATA,NULL,NULL);
+			espeak_Synth(TextBuf,0,0,POS_CHARACTER,0,espeakCHARS_WCHAR | espeakKEEP_NAMEDATA | espeakPHONEMES,NULL,NULL);
 		}
 	}
     return hr;
@@ -661,15 +760,15 @@ STDMETHODIMP CTTSEngObj::GetOutputFormat( const GUID * pTargetFormatId, const WA
 
 int FAR PASCAL CompileDictionary(const char *voice, const char *path_log)
 {//===========================================================
-	FILE *f_log;
+	FILE *f_log3;
 	char fname[120];
 
-	f_log = fopen(path_log,"w");
+	f_log3 = fopen(path_log,"w");
 	sprintf(fname,"%s/",path_install);
 
 	espeak_SetVoiceByName(voice);
-	espeak_CompileDictionary(fname,f_log);
-	fclose(f_log);
+	espeak_CompileDictionary(fname,f_log3,0);
+	fclose(f_log3);
 
 	return(0);
 }
