@@ -39,6 +39,8 @@ static const char *help_text =
 "spoken from stdin, each line separately.\n\n"
 "-a <integer>\n"
 "\t   Amplitude, 0 to 200, default is 100\n"
+"-g <integer>\n"
+"\t   Word gap. Pause between words, units of 10mS at the default speed\n"
 "-l <integer>\n"
 "\t   Line length. If not zero (which is the default), consider\n"
 "\t   lines less than this length as end-of-clause\n"
@@ -63,6 +65,8 @@ static const char *help_text =
 "--punct=\"<characters>\"\n"
 "\t   Speak the names of punctuation characters during speaking.  If\n"
 "\t   =<characters> is omitted, all punctuation is spoken.\n"
+"--split=\"<minutes>\"\n"
+"\t   Starts a new WAV file every <minutes>.  Used with -w\n"
 "--voices=<langauge>\n"
 "\t   List the available voices for the specified language.\n"
 "\t   If <language> is omitted, then list all voices.\n"
@@ -74,8 +78,14 @@ static const char *help_text =
 
 
 int samplerate;
+int quiet = 0;
+unsigned int samples_total = 0;
+unsigned int samples_split = 0;
+unsigned int wavefile_count = 0;
+
 FILE *f_wavfile = NULL;
-char wavefile[160];
+char filetype[5];
+char wavefile[200];
 
 
 int GetFileLength(const char *filename)
@@ -105,12 +115,12 @@ void strncpy0(char *dest, const char *source, int size)
 void DisplayVoices(FILE *f_out, char *language)
 {//============================================
 	int ix;
-	char *p;
+	const char *p;
 	int len;
 	int count;
 	int scores = 0;
 	const espeak_VOICE *v;
-	char *lang_name;
+	const char *lang_name;
 	char age_buf[12];
 	const espeak_VOICE **voices;
 	espeak_VOICE voice_select;
@@ -188,8 +198,8 @@ int OpenWavFile(char *path, int rate)
 {
 	static unsigned char wave_hdr[44] = {
 		'R','I','F','F',0,0,0,0,'W','A','V','E','f','m','t',' ',
-		0x10,0,0,0,1,0,1,0,  0,0,0,0, 0,0,0,0,
-		2,0,0x10,0,'d','a','t','a',  0,0,0,0 };
+		0x10,0,0,0,1,0,1,0,  9,0x3d,0,0,0x12,0x7a,0,0,
+		2,0,0x10,0,'d','a','t','a',  0xff,0xff,0xff,0x7f};
 
 	if(path == NULL)
 		return(2);
@@ -207,7 +217,7 @@ int OpenWavFile(char *path, int rate)
 		fwrite(wave_hdr,1,24,f_wavfile);
 		Write4Bytes(f_wavfile,rate);
 		Write4Bytes(f_wavfile,rate * 2);
-		fwrite(&wave_hdr[32],1,8,f_wavfile);
+		fwrite(&wave_hdr[32],1,12,f_wavfile);
 		return(0);
 	}
 	return(1);
@@ -215,8 +225,8 @@ int OpenWavFile(char *path, int rate)
 
 
 
-static void CloseWavFile(int rate)
-//================================
+static void CloseWavFile()
+//========================
 {
 	unsigned int pos;
 
@@ -238,20 +248,42 @@ static void CloseWavFile(int rate)
 } // end of CloseWavFile
 
 
-
 static int SynthCallback(short *wav, int numsamples, espeak_EVENT *events)
 {//========================================================================
+	char fname[210];
 
-	if(f_wavfile == NULL) return(0);  // -q quiet mode
+	if(quiet) return(0);  // -q quiet mode
 
 	if(wav == NULL)
 	{
-		CloseWavFile(samplerate);
+		CloseWavFile();
 		return(0);
 	}
 
+	if(samples_split > 0)
+	{
+		// start a new WAV file when this limit is reached, at the next sentence boundary
+		while(events->type != 0)
+		{
+			if((events->type == espeakEVENT_SENTENCE) && (samples_total > samples_split))
+			{
+				CloseWavFile();
+				samples_total = 0;
+			}
+			events++;
+		}
+	}
+
+	if(f_wavfile == NULL)
+	{
+		sprintf(fname,"%s_%.2d%s",wavefile,++wavefile_count,filetype);
+		if(OpenWavFile(fname, samplerate) != 0)
+			return(1);
+	}
+
 	if(numsamples > 0)
 	{
+		samples_total += numsamples;
 		fwrite(wav,numsamples*2,1,f_wavfile);
 	}
 	return(0);
@@ -272,10 +304,12 @@ int main (int argc, char **argv)
 			We distinguish them by their indices. */
 		{"help",    no_argument,       0, 'h'},
 		{"stdin",   no_argument,       0, 0x100},
-		{"stdout",  no_argument,       0, 0x101},
+		{"compile-debug", optional_argument, 0, 0x101},
 		{"compile", optional_argument, 0, 0x102},
 		{"punct",   optional_argument, 0, 0x103},
 		{"voices",  optional_argument, 0, 0x104},
+		{"stdout",  no_argument,       0, 0x105},
+		{"split",   optional_argument, 0, 0x106},
 		{0, 0, 0, 0}
 		};
 
@@ -288,7 +322,6 @@ int main (int argc, char **argv)
 	int option_index = 0;
 	int c;
 	int ix;
-	int quiet = 0;
 	int flag_stdin = 0;
 	int flag_compile = 0;
 	int filesize = 0;
@@ -297,6 +330,7 @@ int main (int argc, char **argv)
 	int volume = -1;
 	int speed = -1;
 	int pitch = -1;
+	int wordgap = -1;
 	int option_capitals = -1;
 	int option_punctuation = -1;
 	int option_phonemes = -1;
@@ -319,7 +353,7 @@ int main (int argc, char **argv)
 
 	while(true)
 	{
-		c = getopt_long (argc, argv, "a:bf:hk:l:mp:qs:v:w:xXz",
+		c = getopt_long (argc, argv, "a:bf:g:hk:l:mp:qs:v:w:xXz",
 					long_options, &option_index);
 
 		/* Detect the end of the options. */
@@ -378,6 +412,10 @@ int main (int argc, char **argv)
 			speed = atoi(optarg);
 			break;
 
+		case 'g':
+			wordgap = atoi(optarg);
+			break;
+
 		case 'v':
 			strncpy0(voicename,optarg,sizeof(voicename));
 			break;
@@ -395,14 +433,15 @@ int main (int argc, char **argv)
 			flag_stdin = 1;
 			break;
 
-		case 0x101:		// --stdout
+		case 0x105:		// --stdout
 			option_waveout = 1;
 			strcpy(wavefile,"stdout");
 			break;
 
+		case 0x101:    // --compile-debug
 		case 0x102:		// --compile
 			strncpy0(voicename,optarg,sizeof(voicename));
-			flag_compile = 1;
+			flag_compile = c;
 			quiet = 1;
 			break;
 
@@ -422,6 +461,13 @@ int main (int argc, char **argv)
 			DisplayVoices(stdout,optarg);
 			exit(0);
 
+		case 0x106:   // -- split
+			if(optarg == NULL)
+				samples_split = 30;  // default 30 minutes
+			else
+				samples_split = atoi(optarg);
+			break;
+
 		default:
 			exit(0);
 		}
@@ -432,8 +478,20 @@ int main (int argc, char **argv)
 	{
 		// writing to a file (or no output), we can use synchronous mode
 		samplerate = espeak_Initialize(AUDIO_OUTPUT_SYNCHRONOUS,0,NULL,0);
+		samples_split = (samplerate * samples_split) * 60;
 
 		espeak_SetSynthCallback(SynthCallback);
+		if(samples_split)
+		{
+			char *extn;
+			extn = strrchr(wavefile,'.');
+			if((extn != NULL) && ((wavefile + strlen(wavefile) - extn) <= 4))
+			{
+				strcpy(filetype,extn);
+				*extn = 0;
+			}
+		}
+		else
 		if(option_waveout)
 		{
 			if(OpenWavFile(wavefile,samplerate) != 0)
@@ -459,7 +517,7 @@ int main (int argc, char **argv)
 	if(flag_compile)
 	{
 		// This must be done after the voice is set
-		espeak_CompileDictionary("",stderr);
+		espeak_CompileDictionary("", stderr, flag_compile & 0x1);
 		exit(0);
 	}
 
@@ -474,6 +532,8 @@ int main (int argc, char **argv)
 		espeak_SetParameter(espeakCAPITALS,option_capitals,0);
 	if(option_punctuation >= 0)
 		espeak_SetParameter(espeakPUNCTUATION,option_punctuation,0);
+	if(wordgap >= 0)
+		espeak_SetParameter(espeakWORDGAP,wordgap,0);
 	if(option_linelength > 0)
 		espeak_SetParameter(espeakLINELENGTH,option_linelength,0);
 	if(option_punctuation == 2)
