@@ -50,12 +50,20 @@
 
 
 
+extern void Write4Bytes(FILE *f, int value);
 char path_home[N_PATH_HOME];    // this is the espeak-data directory
 
-char wavefile[120];
+char filetype[5];
+char wavefile[200];
 int (* uri_callback)(int, const char *, const char *) = NULL;
 int (* phoneme_callback)(const char *) = NULL;
 
+FILE *f_wave = NULL;
+int quiet = 0;
+unsigned int samples_total = 0;
+unsigned int samples_split = 0;
+unsigned int wavefile_count = 0;
+int end_of_sentence = 0;
 
 static const char *help_text =
 "\nspeak [options] [\"<words>\"]\n\n"
@@ -65,6 +73,8 @@ static const char *help_text =
 "spoken from stdin, each line separately.\n\n"
 "-a <integer>\n"
 "\t   Amplitude, 0 to 200, default is 100\n"
+"-g <integer>\n"
+"\t   Word gap. Pause between words, units of 10mS at the default speed\n"
 "-l <integer>\n"
 "\t   Line length. If not zero (which is the default), consider\n"
 "\t   lines less than this length as end-of-clause\n"
@@ -81,6 +91,7 @@ static const char *help_text =
 "-q\t   Quiet, don't produce any speech (may be useful with -x)\n"
 "-x\t   Write phoneme mnemonics to stdout\n"
 "-X\t   Write phonemes mnemonics and translation trace to stdout\n"
+"-z\t   No final sentence pause at the end of the text\n"
 "--stdout   Write speech output to stdout\n"
 "--compile=<voice name>\n"
 "\t   Compile the pronunciation rules and dictionary in the current\n"
@@ -88,6 +99,8 @@ static const char *help_text =
 "--punct=\"<characters>\"\n"
 "\t   Speak the names of punctuation characters during speaking.  If\n"
 "\t   =<characters> is omitted, all punctuation is spoken.\n"
+"--split=\"<minutes>\"\n"
+"\t   Starts a new WAV file every <minutes>.  Used with -w\n"
 "--voices=<langauge>\n"
 "\t   List the available voices for the specified language.\n"
 "\t   If <language> is omitted, then list all voices.\n"
@@ -98,9 +111,7 @@ static const char *help_text =
 
 void DisplayVoices(FILE *f_out, char *language);
 
-voice_t voice_data;
 USHORT voice_pcnt[N_PEAKS+1][3];
-voice_t *voice;
 
 
 
@@ -137,12 +148,12 @@ void Free(void *ptr)
 void DisplayVoices(FILE *f_out, char *language)
 {//============================================
 	int ix;
-	char *p;
+	const char *p;
 	int len;
 	int count;
 	int scores = 0;
 	const espeak_VOICE *v;
-	char *lang_name;
+	const char *lang_name;
 	char age_buf[12];
 	const espeak_VOICE **voices;
 	espeak_VOICE voice_select;
@@ -216,19 +227,122 @@ static void PitchAdjust(int pitch_adjustment)
 }  //  end of PitchAdjustment
 
 
+
+
+static int OpenWaveFile(const char *path, int rate)
+//=================================================
+{
+	// Set the length of 0x7fffffff for --stdout
+	// This will be changed to the correct length for -w (write to file)
+	static unsigned char wave_hdr[44] = {
+		'R','I','F','F',0,0,0,0,'W','A','V','E','f','m','t',' ',
+		0x10,0,0,0,1,0,1,0,  9,0x3d,0,0,0x12,0x7a,0,0,
+		2,0,0x10,0,'d','a','t','a',  0xff,0xff,0xff,0x7f};
+
+	if(path == NULL)
+		return(2);
+
+	if(strcmp(path,"stdout")==0)
+		f_wave = stdout;
+	else
+		f_wave = fopen(path,"wb");
+
+	if(f_wave != NULL)
+	{
+		fwrite(wave_hdr,1,24,f_wave);
+		Write4Bytes(f_wave,rate);
+		Write4Bytes(f_wave,rate * 2);
+		fwrite(&wave_hdr[32],1,12,f_wave);
+		return(0);
+	}
+	return(1);
+}   //  end of OpenWaveFile
+
+
+
+
+static void CloseWaveFile()
+//=========================
+{
+   unsigned int pos;
+
+   if((f_wave == NULL) || (f_wave == stdout))
+      return;
+
+   fflush(f_wave);
+   pos = ftell(f_wave);
+
+	fseek(f_wave,4,SEEK_SET);
+	Write4Bytes(f_wave,pos - 8);
+
+	fseek(f_wave,40,SEEK_SET);
+	Write4Bytes(f_wave,pos - 44);
+
+
+   fclose(f_wave);
+   f_wave = NULL;
+
+} // end of CloseWaveFile
+
+
+
+
 void MarkerEvent(int type, unsigned int char_position, int value, unsigned char *out_ptr)
 {//======================================================================================
 // Do nothing in the command-line version.
+	if(type == 2)
+		end_of_sentence = 1;
 }  // end of MarkerEvent
 
 
+static int WavegenFile(void)
+{//=========================
+	int finished;
+	unsigned char wav_outbuf[512];
+	char fname[210];
 
-static void init_path(void)
-{//========================
+	out_ptr = out_start = wav_outbuf;
+	out_end = wav_outbuf + sizeof(wav_outbuf);
+
+	finished = WavegenFill(0);
+
+	if(quiet)
+		return(finished);
+
+	if(f_wave == NULL)
+	{
+		sprintf(fname,"%s_%.2d%s",wavefile,++wavefile_count,filetype);
+		if(OpenWaveFile(fname, samplerate) != 0)
+			return(1);
+	}
+
+	if(end_of_sentence)
+	{
+		end_of_sentence = 0;
+		if((samples_split > 0 ) && (samples_total > samples_split))
+		{
+			CloseWaveFile();
+			samples_total = 0;
+		}
+	}
+
+	if(f_wave != NULL)
+	{
+		samples_total += (out_ptr - wav_outbuf)/2;
+		fwrite(wav_outbuf, 1, out_ptr - wav_outbuf, f_wave);
+	}
+	return(finished);
+}  // end of WavegenFile
+
+
+
+static void init_path(char *argv0)
+{//===============================
 #ifdef PLATFORM_WINDOWS
 	HKEY RegKey;
 	unsigned long size;
 	unsigned long var_type;
+	char *p;
 	char *env;
 	unsigned char buf[sizeof(path_home)-12];
 
@@ -236,9 +350,18 @@ static void init_path(void)
 	{
 		sprintf(path_home,"%s\\espeak-data",env);
 		if(GetFileLength(path_home) == -2)
-			return;   // an espeak-data directory exists 
+			return;   // an espeak-data directory exists in the directory specified by environment variable
 	}
 
+	strcpy(path_home,argv0);
+	if((p = strrchr(path_home,'\\')) != NULL)
+	{
+		strcpy(&p[1],"espeak-data");
+		if(GetFileLength(path_home) == -2)
+			return;   // an espeak-data directory exists in the same directory as the espeak program
+	}
+
+	// otherwise, look in the Windows Registry
 	buf[0] = 0;
 	RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Speech\\Voices\\Tokens\\eSpeak", 0, KEY_READ, &RegKey);
 	size = sizeof(buf);
@@ -250,13 +373,13 @@ static void init_path(void)
 #ifdef PLATFORM_DOS
 		strcpy(path_home,PATH_ESPEAK_DATA);
 #else
-//	char *env;
-//	if((env = getenv("ESPEAK_DATA_PATH")) != NULL)
-//	{
-//		snprintf(path_home,sizeof(path_home),"%s/espeak-data",env);
-//		if(GetFileLength(path_home) == -2)
-//			return;   // an espeak-data directory exists 
-//	}
+	char *env;
+	if((env = getenv("ESPEAK_DATA_PATH")) != NULL)
+	{
+		snprintf(path_home,sizeof(path_home),"%s/espeak-data",env);
+		if(GetFileLength(path_home) == -2)
+			return;   // an espeak-data directory exists 
+	}
 
 	snprintf(path_home,sizeof(path_home),"%s/espeak-data",getenv("HOME"));
 	if(access(path_home,R_OK) != 0)
@@ -295,9 +418,7 @@ static int initialise(void)
 		else
 			fprintf(stderr,"Wrong version of espeak-data 0x%x (expects 0x%x)\n",result,version_phdata);
 	}
-#ifndef PLATFORM_WINDOWS
-	LoadConfig();  // causes problem on Windows, don't know why
-#endif
+	LoadConfig();
 	SetVoiceStack(NULL);
 	SynthesizeInit();
 
@@ -346,10 +467,12 @@ int main (int argc, char **argv)
 			We distinguish them by their indices. */
 		{"help",    no_argument,       0, 'h'},
 		{"stdin",   no_argument,       0, 0x100},
-		{"stdout",  no_argument,       0, 0x101},
+		{"compile-debug", optional_argument, 0, 0x101},
 		{"compile", optional_argument, 0, 0x102},
 		{"punct",   optional_argument, 0, 0x103},
 		{"voices",  optional_argument, 0, 0x104},
+		{"stdout",  no_argument,       0, 0x105},
+		{"split",   optional_argument, 0, 0x106},
 		{0, 0, 0, 0}
 		};
 
@@ -365,8 +488,8 @@ int main (int argc, char **argv)
 	int ix;
 	char *optarg2;
 	int amp = 100;     // default
+	int wordgap = 0;
 	int speaking = 0;
-	int quiet = 0;
 	int flag_stdin = 0;
 	int flag_compile = 0;
 	int pitch_adjustment = 50;
@@ -382,10 +505,12 @@ int main (int argc, char **argv)
 	option_linelength = 0;
 	option_phonemes = 0;
 	option_waveout = 0;
+	option_wordgap = 0;
+	option_endpause = 1;
 	option_multibyte = espeakCHARS_AUTO;  // auto
 	f_trans = stdout;
 
-	init_path();
+	init_path(argv[0]);
 
 #ifdef NEED_GETOPT
 	optind = 1;
@@ -441,7 +566,7 @@ int main (int argc, char **argv)
 #else
 	while(true)
 	{
-		c = getopt_long (argc, argv, "a:bf:hk:l:p:qs:v:w:xXm",
+		c = getopt_long (argc, argv, "a:bf:g:hk:l:p:qs:v:w:xXmz",
 					long_options, &option_index);
 
 		/* Detect the end of the options. */
@@ -504,6 +629,10 @@ int main (int argc, char **argv)
 			speed = atoi(optarg2);
 			break;
 
+		case 'g':
+			wordgap = atoi(optarg2);
+			break;
+
 		case 'v':
 			strncpy0(voicename,optarg2,sizeof(voicename));
 			break;
@@ -513,19 +642,24 @@ int main (int argc, char **argv)
 			strncpy0(wavefile,optarg2,sizeof(wavefile));
 			break;
 
+		case 'z':
+			option_endpause = 0;
+			break;
+
 		case 0x100:		// --stdin
 			flag_stdin = 1;
 			break;
 
-		case 0x101:		// --stdout
+		case 0x105:		// --stdout
 			option_waveout = 1;
 			strcpy(wavefile,"stdout");
 			break;
 
+		case 0x101:    // --compile-debug
 		case 0x102:		// --compile
 			if(optarg2 != NULL)
 				strncpy0(voicename,optarg2,sizeof(voicename));
-			flag_compile = 1;
+			flag_compile = c;
 			break;
 
 		case 0x103:		// --punct
@@ -543,6 +677,13 @@ int main (int argc, char **argv)
 			DisplayVoices(stdout,optarg2);
 			exit(0);
 
+
+		case 0x106:   // -- split
+			if(optarg2 == NULL)
+				samples_split = 30;  // default 30 minutes
+			else
+				samples_split = atoi(optarg2);
+			break;
 		default:
 			exit(0);
 		}
@@ -550,6 +691,32 @@ int main (int argc, char **argv)
 
 	initialise();
 
+
+	if(flag_compile)
+	{
+		LoadVoice(voicename,5);
+
+#ifdef PLATFORM_DOS
+		char path_dsource[sizeof(path_home)+20];
+		strcpy(path_dsource,path_home);
+		path_dsource[strlen(path_home)-11] = 0;  // remove "espeak-data" from the end
+		strcat(path_dsource,"dictsource\\");
+		CompileDictionary(path_dsource,dictionary_name,NULL,NULL, flag_compile & 0x1);
+#else
+#ifdef PLATFORM_WINDOWS
+		char path_dsource[sizeof(path_home)+20];
+		strcpy(path_dsource,path_home);
+		path_dsource[strlen(path_home)-11] = 0;  // remove "espeak-data" from the end
+		strcat(path_dsource,"dictsource\\");
+		CompileDictionary(path_dsource,dictionary_name,NULL,NULL, flag_compile & 0x1);
+#else
+		CompileDictionary(NULL,dictionary_name,NULL,NULL, flag_compile & 0x1);
+#endif
+#endif
+		exit(0);
+	}
+
+
 	if(voicename[0] == 0)
 		strcpy(voicename,"default");
 
@@ -559,39 +726,17 @@ int main (int argc, char **argv)
 		exit(2);
 	}
 
-
-	if(flag_compile)
-	{
-#ifdef PLATFORM_DOS
-		char path_dsource[sizeof(path_home)+20];
-		strcpy(path_dsource,path_home);
-		path_dsource[strlen(path_home)-11] = 0;  // remove "espeak-data" from the end
-		strcat(path_dsource,"dictsource\\");
-		CompileDictionary(path_dsource,dictionary_name,NULL,NULL);
-#else
-#ifdef PLATFORM_WINDOWS
-		char path_dsource[sizeof(path_home)+20];
-		strcpy(path_dsource,path_home);
-		path_dsource[strlen(path_home)-11] = 0;  // remove "espeak-data" from the end
-		strcat(path_dsource,"dictsource\\");
-		CompileDictionary(path_dsource,dictionary_name,NULL,NULL);
-#else
-		CompileDictionary(NULL,dictionary_name,NULL,NULL);
-#endif
-#endif
-		exit(0);
-	}
-
 	SetParameter(espeakRATE,speed,0);
 	SetParameter(espeakVOLUME,amp,0);
 	SetParameter(espeakCAPITALS,option_capitals,0);
 	SetParameter(espeakPUNCTUATION,option_punctuation,0);
+	SetParameter(espeakWORDGAP,wordgap,0);
 
 	if(pitch_adjustment != 50)
 	{
 		PitchAdjust(pitch_adjustment);
 	}
-	WavegenSetVoice(voice);
+	DoVoiceChange(voice);
 
 	if(filename[0]==0)
 	{
@@ -630,6 +775,20 @@ int main (int argc, char **argv)
 		else
 		{
 			// write sound output to a WAV file
+			samples_split = (samplerate * samples_split) * 60;
+
+			if(samples_split)
+			{
+				// don't open the wav file until we start generating speech
+				char *extn;
+				extn = strrchr(wavefile,'.');
+				if((extn != NULL) && ((wavefile + strlen(wavefile) - extn) <= 4))
+				{
+					strcpy(filetype,extn);
+					*extn = 0;
+				}
+			}
+			else
 			if(OpenWaveFile(wavefile,samplerate) != 0)
 			{
 				fprintf(stderr,"Can't write to output file '%s'\n'",wavefile);
@@ -640,18 +799,22 @@ int main (int argc, char **argv)
 		InitText(0);
 		SpeakNextClause(f_text,p_text,0);
 
+		ix = 1;
 		for(;;)
 		{
 			if(WavegenFile() != 0)
 			{
-				break;   // finished, wavegen command queue is empty
+				if(ix == 0)
+					break;   // finished, wavegen command queue is empty
 			}
 
 			if(Generate(phoneme_list,&n_phoneme_list,1)==0)
-				SpeakNextClause(NULL,NULL,1);
+			{
+				ix = SpeakNextClause(NULL,NULL,1);
+			}
 		}
 
-		CloseWaveFile(samplerate);
+		CloseWaveFile();
 	}
 	else
 	{
