@@ -1,10 +1,10 @@
 /***************************************************************************
- *   Copyright (C) 2005,2006 by Jonathan Duddington                        *
- *   jonsd@users.sourceforge.net                                           *
+ *   Copyright (C) 2005 to 2007 by Jonathan Duddington                     *
+ *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation; either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
@@ -13,10 +13,10 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   along with this program; if not, write see:                           *
+ *               <http://www.gnu.org/licenses/>.                           *
  ***************************************************************************/
+
 #include "StdAfx.h"
 
 #include "stdio.h"
@@ -26,18 +26,19 @@
 #include "wchar.h"
 #include "locale.h"
 #include <assert.h>
+#include <time.h>
 
 #include "speech.h"
 
-#include "sys/stat.h"
+#include <sys/stat.h>
 #ifndef PLATFORM_WINDOWS
 #include <unistd.h>
 #endif
 
 #include "speak_lib.h"
-#include "voice.h"
 #include "phoneme.h"
 #include "synthesize.h"
+#include "voice.h"
 #include "translate.h"
 #include "debug.h"
 
@@ -46,8 +47,8 @@
 #include "wave.h"
 
 unsigned char *outbuf=NULL;
+extern espeak_VOICE voice_selected;
 
-int outbuf_size=0;
 espeak_EVENT *event_list=NULL;
 int event_list_ix=0;
 int n_event_list;
@@ -57,13 +58,12 @@ void* my_audio=NULL;
 static unsigned int my_unique_identifier=0;
 static void* my_user_data=NULL;
 static espeak_AUDIO_OUTPUT my_mode=AUDIO_OUTPUT_SYNCHRONOUS;
-static t_espeak_callback* synth_callback = NULL;
+static int synchronous_mode = 1;
+t_espeak_callback* synth_callback = NULL;
 int (* uri_callback)(int, const char *, const char *) = NULL;
+int (* phoneme_callback)(const char *) = NULL;
 
-char path_home[120];
-
-voice_t voicedata;
-voice_t *voice = &voicedata;
+char path_home[N_PATH_HOME];   // this is the espeak-data directory
 
 
 #ifdef USE_ASYNC
@@ -71,13 +71,13 @@ voice_t *voice = &voicedata;
 static int dispatch_audio(short* outbuf, int length, espeak_EVENT* event)
 {//======================================================================
 	ENTER("dispatch_audio");
-
+	
 	int a_wave_can_be_played = fifo_is_command_enabled();
-
+	
 	SHOW("*** dispatch_audio > uid=%d, [write=%p (%d bytes)], sample=%d, a_wave_can_be_played = %d\n", 
-       (event) ? event->unique_identifier : 0, wave_test_get_write_buffer(), 2*length, 
-       (event) ? event->sample : 0,
-       a_wave_can_be_played);
+			(event) ? event->unique_identifier : 0, wave_test_get_write_buffer(), 2*length, 
+			(event) ? event->sample : 0,
+			a_wave_can_be_played);
 
 
 	switch(my_mode)
@@ -119,6 +119,7 @@ static int dispatch_audio(short* outbuf, int length, espeak_EVENT* event)
 		break;
 
 	case AUDIO_OUTPUT_SYNCHRONOUS:
+	case AUDIO_OUTPUT_SYNCH_PLAYBACK:
 		break;
 	}
 
@@ -167,20 +168,6 @@ static int create_events(short* outbuf, int length, espeak_EVENT* event, uint32_
 	return finished;
 }
 
-static void select_output(espeak_AUDIO_OUTPUT output_type)
-{//=======================================================
-	my_mode = output_type;
-	my_audio = NULL;
-
-	if(my_mode == AUDIO_OUTPUT_PLAYBACK)
-	{
-		wave_init();
-		wave_set_callback_is_output_enabled( fifo_is_command_enabled);
-		my_audio = wave_open("alsa");
-		event_init(synth_callback);
-	}
-}   // end of select_output
-
 
 int sync_espeak_terminated_msg( uint unique_identifier, void* user_data)
 {//=====================================================================
@@ -194,6 +181,8 @@ int sync_espeak_terminated_msg( uint unique_identifier, void* user_data)
 	event_list[0].unique_identifier = unique_identifier;
 	event_list[0].user_data = user_data;
 	event_list[1].type = espeakEVENT_LIST_TERMINATED;
+	event_list[1].unique_identifier = unique_identifier;
+	event_list[1].user_data = user_data;
 
 	if (my_mode==AUDIO_OUTPUT_PLAYBACK)
 	{
@@ -221,33 +210,69 @@ int sync_espeak_terminated_msg( uint unique_identifier, void* user_data)
 #endif
 
 
+static void select_output(espeak_AUDIO_OUTPUT output_type)
+{//=======================================================
+	my_mode = output_type;
+	my_audio = NULL;
+	synchronous_mode = 1;
+ 	option_waveout = 1;   // inhibit portaudio callback from wavegen.cpp
+
+	switch(my_mode)
+	{
+	case AUDIO_OUTPUT_PLAYBACK:
+		synchronous_mode = 0;
+#ifdef USE_ASYNC
+		wave_init();
+		wave_set_callback_is_output_enabled( fifo_is_command_enabled);
+		my_audio = wave_open("alsa");
+		event_init();
+#endif
+		break;
+
+	case AUDIO_OUTPUT_RETRIEVAL:
+		synchronous_mode = 0;
+		break;
+
+	case AUDIO_OUTPUT_SYNCHRONOUS:
+		break;
+
+	case AUDIO_OUTPUT_SYNCH_PLAYBACK:
+		option_waveout = 0;
+		WavegenInitSound();
+		break;
+	}
+}   // end of select_output
+
+
+
+
 int GetFileLength(const char *filename)
 {//====================================
-  struct stat statbuf;
-
-  if(stat(filename,&statbuf) != 0)
-    return(0);
-
-  if((statbuf.st_mode & S_IFMT) == S_IFDIR)
-    //	if(S_ISDIR(statbuf.st_mode))
-    return(-2);  // a directory
-
-  return(statbuf.st_size);
+	struct stat statbuf;
+	
+	if(stat(filename,&statbuf) != 0)
+		return(0);
+	
+	if((statbuf.st_mode & S_IFMT) == S_IFDIR)
+		//	if(S_ISDIR(statbuf.st_mode))
+		return(-2);  // a directory
+	
+	return(statbuf.st_size);
 }  // end of GetFileLength
 
 
 char *Alloc(int size)
 {//==================
-  char *p;
-  if((p = (char *)malloc(size)) == NULL)
-    fprintf(stderr,"Can't allocate memory\n");
-  return(p);
+	char *p;
+	if((p = (char *)malloc(size)) == NULL)
+		fprintf(stderr,"Can't allocate memory\n");  // I was told that size+1 fixes a crash on 64-bit systems
+	return(p);
 }
 
 void Free(void *ptr)
 {//=================
-  if(ptr != NULL)
-    free(ptr);
+	if(ptr != NULL)
+		free(ptr);
 }
 
 
@@ -255,21 +280,50 @@ void Free(void *ptr)
 static void init_path(const char *path)
 {//====================================
 #ifdef PLATFORM_WINDOWS
+	HKEY RegKey;
+	unsigned long size;
+	unsigned long var_type;
+	char *env;
+	unsigned char buf[sizeof(path_home)-13];
+
 	if(path != NULL)
 	{
 		sprintf(path_home,"%s/espeak-data",path);
 		return;
 	}
 
-	strcpy(path_home,"c:\\Program Files\\espeak\\espeak-data");
+	if((env = getenv("ESPEAK_DATA_PATH")) != NULL)
+	{
+		sprintf(path_home,"%s/espeak-data",env);
+		if(GetFileLength(path_home) == -2)
+			return;   // an espeak-data directory exists 
+	}
+
+	buf[0] = 0;
+	RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Speech\\Voices\\Tokens\\eSpeak", 0, KEY_READ, &RegKey);
+	size = sizeof(buf);
+	var_type = REG_SZ;
+	RegQueryValueEx(RegKey, "path", 0, &var_type, buf, &size);
+
+	sprintf(path_home,"%s\\espeak-data",buf);
 
 #else
+	char *env;
+
 	if(path != NULL)
 	{
 		snprintf(path_home,sizeof(path_home),"%s/espeak-data",path);
 		return;
 	}
 
+	// check for environment variable
+	if((env = getenv("ESPEAK_DATA_PATH")) != NULL)
+	{
+		snprintf(path_home,sizeof(path_home),"%s/espeak-data",env);
+		if(GetFileLength(path_home) == -2)
+			return;   // an espeak-data directory exists 
+	}
+
 	snprintf(path_home,sizeof(path_home),"%s/espeak-data",getenv("HOME"));
 	if(access(path_home,R_OK) != 0)
 	{
@@ -283,18 +337,18 @@ static int initialise(void)
 	int param;
 	int result;
 
-#ifndef __WIN32__
-	LoadConfig();  // causes problem on Windows, don't know why
-#endif
+	LoadConfig();
 	WavegenInit(22050,0);   // 22050
 	if((result = LoadPhData()) != 1)
 	{
 		if(result == -1)
 			fprintf(stderr,"Failed to load espeak-data\n");
 		else
-			fprintf(stderr,"Wrong version of espeak-data 0x%x (0x%x)\n",result,version_phdata);
+			fprintf(stderr,"Wrong version of espeak-data 0x%x (expects 0x%x)\n",result,version_phdata);
 	}
 
+	memset(&voice_selected,0,sizeof(voice_selected));
+	SetVoiceStack(NULL);
 	SynthesizeInit();
 	InitNamedata();
 
@@ -305,7 +359,6 @@ static int initialise(void)
 }
 
 
-
 static espeak_ERROR Synthesize(unsigned int unique_identifier, const void *text, int flags)
 {//========================================================================================
 	// Fill the buffer with output sound
@@ -313,7 +366,9 @@ static espeak_ERROR Synthesize(unsigned int unique_identifier, const void *text,
 	int length;
 	int finished = 0;
 	int count_buffers = 0;
+#ifdef USE_ASYNC
 	uint32_t a_write_pos=0;
+#endif
 
 #ifdef DEBUG_ENABLED
 	if (text)
@@ -341,11 +396,34 @@ static espeak_ERROR Synthesize(unsigned int unique_identifier, const void *text,
 
 	if(translator == NULL)
 	{
-		LoadVoice("default",0);
+		SetVoiceByName("default");
 	}
 
 	SpeakNextClause(NULL,text,0);
 
+	if(my_mode == AUDIO_OUTPUT_SYNCH_PLAYBACK)
+	{
+		for(;;)
+		{
+#ifdef PLATFORM_WINDOWS
+			Sleep(300);   // 0.3s
+#else
+#ifdef USE_NANOSLEEP
+			struct timespec period;
+			struct timespec remaining;
+			period.tv_sec = 0;
+			period.tv_nsec = 300000000;  // 0.3 sec
+			nanosleep(&period,&remaining);
+#else
+			sleep(1);
+#endif
+#endif
+			if(SynthOnTimer() != 0)
+				break;
+		}
+		return(EE_OK);
+	}
+
 	for(;;)
 	{
 		SHOW("Synthesize > %s\n","for (next)");
@@ -357,6 +435,8 @@ static espeak_ERROR Synthesize(unsigned int unique_identifier, const void *text,
 		length = (out_ptr - outbuf)/2;
 		count_samples += length;
 		event_list[event_list_ix].type = espeakEVENT_LIST_TERMINATED; // indicates end of event list
+		event_list[event_list_ix].unique_identifier = my_unique_identifier;
+		event_list[event_list_ix].user_data = my_user_data;
 
 		count_buffers++;
 		if (my_mode==AUDIO_OUTPUT_PLAYBACK)
@@ -382,6 +462,11 @@ static espeak_ERROR Synthesize(unsigned int unique_identifier, const void *text,
 			{
 				// don't process the next clause until the previous clause has finished generating speech.
 				// This ensures that <audio> tag (which causes end-of-clause) is at a sound buffer boundary
+
+				event_list[0].type = espeakEVENT_LIST_TERMINATED;
+				event_list[0].unique_identifier = my_unique_identifier;
+				event_list[0].user_data = my_user_data;
+
 				if(SpeakNextClause(NULL,NULL,1)==0)
 				{
 #ifdef USE_ASYNC
@@ -417,34 +502,34 @@ static const char* label[] = {
 
 void MarkerEvent(int type, unsigned int char_position, int value, unsigned char *out_ptr)
 {//======================================================================================
-  // type: 1=word, 2=sentence, 3=named mark, 4=play audio, 5=end
-  ENTER("MarkerEvent");
-  espeak_EVENT *ep;
-  double time;
-
-  if((event_list == NULL) || (event_list_ix >= (n_event_list-2)))
-    return;
-
-  ep = &event_list[event_list_ix++];
-  ep->type = (espeak_EVENT_TYPE)type;
-  ep->unique_identifier = my_unique_identifier;
-  ep->user_data = my_user_data;
-  ep->text_position = char_position & 0xffff;
-  ep->length = char_position >> 24;
-
-  time = (double(count_samples + (out_ptr - outbuf)/2)*1000.0)/samplerate;
-  ep->audio_position = int(time);
-  ep->sample = (count_samples + (out_ptr - outbuf)/2);
-
-  SHOW("MarkerEvent > count_samples=%d, out_ptr=%x, outbuf=0x%x\n",count_samples, out_ptr, outbuf);
-  SHOW("*** MarkerEvent > type=%s, uid=%d, text_pos=%d, length=%d, audio_position=%d, sample=%d\n",
-       label[ep->type], ep->unique_identifier, ep->text_position, ep->length,
-       ep->audio_position, ep->sample);
-
-  if((type == espeakEVENT_MARK) || (type == espeakEVENT_PLAY))
-    ep->id.name = &namedata[value];
-  else
-    ep->id.number = value;
+	// type: 1=word, 2=sentence, 3=named mark, 4=play audio, 5=end
+	ENTER("MarkerEvent");
+	espeak_EVENT *ep;
+	double time;
+	
+	if((event_list == NULL) || (event_list_ix >= (n_event_list-2)))
+		return;
+	
+	ep = &event_list[event_list_ix++];
+	ep->type = (espeak_EVENT_TYPE)type;
+	ep->unique_identifier = my_unique_identifier;
+	ep->user_data = my_user_data;
+	ep->text_position = char_position & 0xffff;
+	ep->length = char_position >> 24;
+	
+	time = (double(count_samples + (out_ptr - out_start)/2)*1000.0)/samplerate;
+	ep->audio_position = int(time);
+	ep->sample = (count_samples + (out_ptr - out_start)/2);
+	
+	SHOW("MarkerEvent > count_samples=%d, out_ptr=%x, out_start=0x%x\n",count_samples, out_ptr, out_start);
+	SHOW("*** MarkerEvent > type=%s, uid=%d, text_pos=%d, length=%d, audio_position=%d, sample=%d\n",
+			label[ep->type], ep->unique_identifier, ep->text_position, ep->length,
+			ep->audio_position, ep->sample);
+	
+	if((type == espeakEVENT_MARK) || (type == espeakEVENT_PLAY))
+		ep->id.name = &namedata[value];
+	else
+		ep->id.number = value;
 }  //  end of MarkerEvent
 
 
@@ -454,40 +539,43 @@ espeak_ERROR sync_espeak_Synth(unsigned int unique_identifier, const void *text,
 		      unsigned int position, espeak_POSITION_TYPE position_type, 
 		      unsigned int end_position, unsigned int flags, void* user_data)
 {//===========================================================================
-  ENTER("sync_espeak_Synth");
-
-  SHOW("sync_espeak_Synth > position=%d, position_type=%d, end_position=%d, flags=%d, user_data=0x%x, text=%s\n", position, position_type, end_position, flags, user_data, text);
-
-	espeak_ERROR aStatus;
-
-  InitText(flags);
-  my_unique_identifier = unique_identifier;
-  my_user_data = user_data;
-
-  switch(position_type)
-    {
-    case POS_CHARACTER:
-      skip_characters = position;
-      break;
-
-    case POS_WORD:
-      skip_words = position;
-      break;
-
-    case POS_SENTENCE:
-      skip_sentences = position;
-      break;
-
-    }
-  if(skip_characters || skip_words || skip_sentences)
-    skipping_text = 1;
-
-  end_character_position = end_position;
-
-  aStatus = Synthesize(unique_identifier, text, flags);
-
-  SHOW_TIME("LEAVE sync_espeak_Synth");
-  return aStatus;
+	ENTER("sync_espeak_Synth");
+	
+	SHOW("sync_espeak_Synth > position=%d, position_type=%d, end_position=%d, flags=%d, user_data=0x%x, text=%s\n", position, position_type, end_position, flags, user_data, text);
+	
+		espeak_ERROR aStatus;
+	
+	InitText(flags);
+	my_unique_identifier = unique_identifier;
+	my_user_data = user_data;
+	
+	switch(position_type)
+		{
+		case POS_CHARACTER:
+			skip_characters = position;
+			break;
+	
+		case POS_WORD:
+			skip_words = position;
+			break;
+	
+		case POS_SENTENCE:
+			skip_sentences = position;
+			break;
+	
+		}
+	if(skip_characters || skip_words || skip_sentences)
+		skipping_text = 1;
+	
+	end_character_position = end_position;
+	
+	aStatus = Synthesize(unique_identifier, text, flags);
+	#ifdef USE_ASYNC
+	wave_flush(my_audio);
+	#endif
+	
+	SHOW_TIME("LEAVE sync_espeak_Synth");
+	return aStatus;
 }  //  end of sync_espeak_Synth
 
 
@@ -498,59 +586,70 @@ espeak_ERROR sync_espeak_Synth_Mark(unsigned int unique_identifier, const void *
 			   unsigned int flags, void* user_data)
 {//=========================================================================
 	espeak_ERROR aStatus;
-
-  InitText(flags);
-
-  my_unique_identifier = unique_identifier;
-  my_user_data = user_data;
-
-  if(index_mark != NULL)
-    {
-      strncpy0(skip_marker, index_mark, sizeof(skip_marker));
-      skipping_text = 1;
-    }
-
-  end_character_position = end_position;
-
-
-  aStatus = Synthesize(unique_identifier, text, flags | espeakSSML);
-  SHOW_TIME("LEAVE sync_espeak_Synth_Mark");
-
-  return (aStatus);
+	
+	InitText(flags);
+	
+	my_unique_identifier = unique_identifier;
+	my_user_data = user_data;
+	
+	if(index_mark != NULL)
+		{
+		strncpy0(skip_marker, index_mark, sizeof(skip_marker));
+		skipping_text = 1;
+		}
+	
+	end_character_position = end_position;
+	
+	
+	aStatus = Synthesize(unique_identifier, text, flags | espeakSSML);
+	SHOW_TIME("LEAVE sync_espeak_Synth_Mark");
+	
+	return (aStatus);
 }  //  end of sync_espeak_Synth_Mark
 
 
 
 void sync_espeak_Key(const char *key)
 {//==================================
-  // symbolic name, symbolicname_character  - is there a system resource of symbolic names per language?
-  my_unique_identifier = 0;
-  my_user_data = NULL;
-  Synthesize(0, key,0);  // for now, just say the key name as passed
+	// symbolic name, symbolicname_character  - is there a system resource of symbolic names per language?
+	int letter;
+	int ix;
+
+	ix = utf8_in(&letter,key,0);
+	if(key[ix] == 0)
+	{
+		// a single character
+		sync_espeak_Char(letter);
+		return;
+	}
+
+	my_unique_identifier = 0;
+	my_user_data = NULL;
+	Synthesize(0, key,0);   // speak key as a text string
 }
 
 
 void sync_espeak_Char(wchar_t character)
 {//=====================================
-  // is there a system resource of character names per language?
-  char buf[80];
-  my_unique_identifier = 0;
-  my_user_data = NULL;
+	// is there a system resource of character names per language?
+	char buf[80];
+	my_unique_identifier = 0;
+	my_user_data = NULL;
 	
-  sprintf(buf,"<say-as interpret-as=\"tts:char\">&#%d;</say-as>",character);
-  Synthesize(0, buf,espeakSSML);
+	sprintf(buf,"<say-as interpret-as=\"tts:char\">&#%d;</say-as>",character);
+	Synthesize(0, buf,espeakSSML);
 }
 
 
 
 void sync_espeak_SetPunctuationList(const wchar_t *punctlist)
 {//==========================================================
-  // Set the list of punctuation which are spoken for "some".
-  my_unique_identifier = 0;
-  my_user_data = NULL;
-
-  wcsncpy(option_punctlist, punctlist, N_PUNCTLIST);
-  option_punctlist[N_PUNCTLIST-1] = 0;
+	// Set the list of punctuation which are spoken for "some".
+	my_unique_identifier = 0;
+	my_user_data = NULL;
+	
+	wcsncpy(option_punctlist, punctlist, N_PUNCTLIST);
+	option_punctlist[N_PUNCTLIST-1] = 0;
 }  //  end of sync_espeak_SetPunctuationList
 
 
@@ -559,67 +658,80 @@ void sync_espeak_SetPunctuationList(const wchar_t *punctlist)
 #pragma GCC visibility push(default)
 
 
-extern "C" void espeak_SetSynthCallback(t_espeak_callback* SynthCallback)
+ESPEAK_API void espeak_SetSynthCallback(t_espeak_callback* SynthCallback)
 {//======================================================================
-  ENTER("espeak_SetSynthCallback");
-  synth_callback = SynthCallback;
+	ENTER("espeak_SetSynthCallback");
+	synth_callback = SynthCallback;
+#ifdef USE_ASYNC
+	event_set_callback(synth_callback);
+#endif
 }
 
-extern "C" void espeak_SetUriCallback(int (* UriCallback)(int, const char*, const char *))
+ESPEAK_API void espeak_SetUriCallback(int (* UriCallback)(int, const char*, const char *))
 {//=======================================================================================
-  ENTER("espeak_SetUriCallback");
-  uri_callback = UriCallback;
+	ENTER("espeak_SetUriCallback");
+	uri_callback = UriCallback;
 }
 
-extern "C" int espeak_Initialize(espeak_AUDIO_OUTPUT output_type, int buf_length, const char *path)
-{//================================================================================================
-  ENTER("espeak_Initialize");
-  int param;
 
-  // It seems that the wctype functions don't work until the locale has been set
-  // to something other than the default "C".  Then, not only Latin1 but also the
-  // other characters give the correct results with iswalpha() etc.
+ESPEAK_API void espeak_SetPhonemeCallback(int (* PhonemeCallback)(const char*))
+{//===========================================================================
+	phoneme_callback = PhonemeCallback;
+}
+
+ESPEAK_API int espeak_Initialize(espeak_AUDIO_OUTPUT output_type, int buf_length, const char *path, int options)
+{//=============================================================================================================
+ENTER("espeak_Initialize");
+	int param;
+
+	// It seems that the wctype functions don't work until the locale has been set
+	// to something other than the default "C".  Then, not only Latin1 but also the
+	// other characters give the correct results with iswalpha() etc.
 #ifdef PLATFORM_RISCOS
-  setlocale(LC_CTYPE,"ISO8859-1");
+	setlocale(LC_CTYPE,"ISO8859-1");
 #else
-  if(setlocale(LC_CTYPE,"en_US.UTF-8") == NULL)
-    setlocale(LC_CTYPE,"german");
+	if(setlocale(LC_CTYPE,"en_US.UTF-8") == NULL)
+	{
+		if(setlocale(LC_CTYPE,"UTF-8") == NULL)
+			setlocale(LC_CTYPE,"");
+	}
 #endif
+	
+	init_path(path);
+	initialise();
+	select_output(output_type);
+	
+	// buflength is in mS, allocate 2 bytes per sample
+	if(buf_length == 0)
+		buf_length = 200;
+	outbuf_size = (buf_length * samplerate)/500;
+	outbuf = (unsigned char*)realloc(outbuf,outbuf_size);
+	if((out_start = outbuf) == NULL)
+		return(EE_INTERNAL_ERROR);
+	
+	// allocate space for event list.  Allow 200 events per second.
+	// Add a constant to allow for very small buf_length
+	n_event_list = (buf_length*200)/1000 + 20;
+	if((event_list = (espeak_EVENT *)realloc(event_list,sizeof(espeak_EVENT) * n_event_list)) == NULL)
+		return(EE_INTERNAL_ERROR);
+	
+	option_phonemes = 0;
+	option_phoneme_events = (options & 1);
 
-  init_path(path);
-  initialise();
+	SetVoiceByName("default");
+	
+	for(param=0; param<N_SPEECH_PARAM; param++)
+		param_stack[0].parameter[param] = param_defaults[param];
+	
+	SetParameter(espeakRATE,170,0);
+	SetParameter(espeakVOLUME,100,0);
+	SetParameter(espeakCAPITALS,option_capitals,0);
+	SetParameter(espeakPUNCTUATION,option_punctuation,0);
+	SetParameter(espeakWORDGAP,0,0);
+	DoVoiceChange(voice);
+	
 #ifdef USE_ASYNC
-  select_output(output_type);
-#else
-  my_mode = output_type;
-#endif
-
-  // buflength is in mS, allocate 2 bytes per sample
-  if(buf_length == 0)
-    buf_length = 200;
-  outbuf_size = (buf_length * samplerate)/500;
-  outbuf = (unsigned char*)realloc(outbuf,outbuf_size);
-
-  // allocate space for event list.  Allow 500 events per minute
-  n_event_list = (buf_length*500)/1000;
-  event_list = (espeak_EVENT *)realloc(event_list,sizeof(espeak_EVENT) * n_event_list);
-
-  option_waveout = 1;
-  option_phonemes = 0;
-
-  LoadVoice("default",0);
-
-  for(param=0; param<N_SPEECH_PARAM; param++)
-    param_stack[0].parameter[param] = param_defaults[param];
-
-  SetParameter(espeakRATE,170,0);
-  SetParameter(espeakVOLUME,100,0);
-  SetParameter(espeakCAPITALS,option_capitals,0);
-  SetParameter(espeakPUNCTUATION,option_punctuation,0);
-  WavegenSetVoice(voice);
-
-#ifdef USE_ASYNC
-  fifo_init();
+	fifo_init();
 #endif
 
   return(samplerate);
@@ -627,7 +739,7 @@ extern "C" int espeak_Initialize(espeak_AUDIO_OUTPUT output_type, int buf_length
 
 
 
-extern "C" espeak_ERROR espeak_Synth(const void *text, size_t size, 
+ESPEAK_API espeak_ERROR espeak_Synth(const void *text, size_t size, 
 				     unsigned int position, 
 				     espeak_POSITION_TYPE position_type,
 				     unsigned int end_position, unsigned int flags, 
@@ -645,7 +757,7 @@ extern "C" espeak_ERROR espeak_Synth(const void *text, size_t size,
 	}
 	*unique_identifier = 0;
 
-	if(my_mode == AUDIO_OUTPUT_SYNCHRONOUS)
+	if(synchronous_mode)
 	{
 		return(sync_espeak_Synth(0,text,size,position,position_type,end_position,flags,user_data));
 	}
@@ -683,7 +795,7 @@ extern "C" espeak_ERROR espeak_Synth(const void *text, size_t size,
 
 
 
-extern "C" espeak_ERROR espeak_Synth_Mark(const void *text, size_t size, 
+ESPEAK_API espeak_ERROR espeak_Synth_Mark(const void *text, size_t size, 
 					  const char *index_mark, 
 					  unsigned int end_position, 
 					  unsigned int flags, 
@@ -702,53 +814,53 @@ extern "C" espeak_ERROR espeak_Synth_Mark(const void *text, size_t size,
 	}
 	*unique_identifier = 0;
 
-	if(my_mode == AUDIO_OUTPUT_SYNCHRONOUS)
+	if(synchronous_mode)
 	{
 		return(sync_espeak_Synth_Mark(0,text,size,index_mark,end_position,flags,user_data));
 	}
 
 #ifdef USE_ASYNC
-  // Create the mark command
-  t_espeak_command* c1 = create_espeak_mark(text, size, index_mark, end_position, 
-					   flags, user_data);
-
-  // Retrieve the unique identifier
-  *unique_identifier = c1->u.my_mark.unique_identifier;
-
-  // Create the "terminated msg" command (same uid)
-  t_espeak_command* c2 = create_espeak_terminated_msg(*unique_identifier, user_data);
-
-  // Try to add these 2 commands (single transaction)
-  if (c1 && c2)
-    {
-      a_error = fifo_add_commands(c1, c2);
-      if (a_error != EE_OK)
+	// Create the mark command
+	t_espeak_command* c1 = create_espeak_mark(text, size, index_mark, end_position, 
+						flags, user_data);
+	
+	// Retrieve the unique identifier
+	*unique_identifier = c1->u.my_mark.unique_identifier;
+	
+	// Create the "terminated msg" command (same uid)
+	t_espeak_command* c2 = create_espeak_terminated_msg(*unique_identifier, user_data);
+	
+	// Try to add these 2 commands (single transaction)
+	if (c1 && c2)
 	{
-	  delete_espeak_command(c1);
-	  delete_espeak_command(c2);
-	  c1=c2=NULL;
+		a_error = fifo_add_commands(c1, c2);
+		if (a_error != EE_OK)
+		{
+			delete_espeak_command(c1);
+			delete_espeak_command(c2);
+			c1=c2=NULL;
+		}
+	}
+	else
+	{
+		delete_espeak_command(c1);
+		delete_espeak_command(c2);
 	}
-    }
-  else
-    {
-      delete_espeak_command(c1);
-      delete_espeak_command(c2);
-    }
 
 #endif
-  return a_error;
+	return a_error;
 }  //  end of espeak_Synth_Mark
 
 
 
-extern "C" espeak_ERROR espeak_Key(const char *key)
+ESPEAK_API espeak_ERROR espeak_Key(const char *key)
 {//================================================
 	ENTER("espeak_Key");
 	// symbolic name, symbolicname_character  - is there a system resource of symbolicnames per language
 
 	espeak_ERROR a_error = EE_OK;
 
-	if(my_mode == AUDIO_OUTPUT_SYNCHRONOUS)
+	if(synchronous_mode)
 	{
 		sync_espeak_Key(key);
 		return(EE_OK);
@@ -767,7 +879,7 @@ extern "C" espeak_ERROR espeak_Key(const char *key)
 }
 
 
-extern "C" espeak_ERROR espeak_Char(wchar_t character)
+ESPEAK_API espeak_ERROR espeak_Char(wchar_t character)
 {//===========================================
   ENTER("espeak_Char");
   // is there a system resource of character names per language?
@@ -775,7 +887,7 @@ extern "C" espeak_ERROR espeak_Char(wchar_t character)
 #ifdef USE_ASYNC
 	espeak_ERROR a_error;
 
-	if(my_mode == AUDIO_OUTPUT_SYNCHRONOUS)
+	if(synchronous_mode)
 	{
 		sync_espeak_Char(character);
 		return(EE_OK);
@@ -795,14 +907,14 @@ extern "C" espeak_ERROR espeak_Char(wchar_t character)
 }
 
 
-extern "C" espeak_ERROR espeak_SetVoiceByName(const char *name)
+ESPEAK_API espeak_ERROR espeak_SetVoiceByName(const char *name)
 {//===================================================
   ENTER("espeak_SetVoiceByName");
 
 #ifdef USE_ASYNC
 	espeak_ERROR a_error;
 
-	if(my_mode == AUDIO_OUTPUT_SYNCHRONOUS)
+	if(synchronous_mode)
 	{
 		return(SetVoiceByName(name));
 	}
@@ -821,14 +933,14 @@ extern "C" espeak_ERROR espeak_SetVoiceByName(const char *name)
 
 
 
-extern "C" espeak_ERROR espeak_SetVoiceByProperties(espeak_VOICE *voice_selector)
+ESPEAK_API espeak_ERROR espeak_SetVoiceByProperties(espeak_VOICE *voice_selector)
 {//=====================================================================
   ENTER("espeak_SetVoiceByProperties");
 
 #ifdef USE_ASYNC
 	espeak_ERROR a_error;
 
-	if(my_mode == AUDIO_OUTPUT_SYNCHRONOUS)
+	if(synchronous_mode)
 	{
 		return(SetVoiceByProperties(voice_selector));
 	}
@@ -846,29 +958,29 @@ extern "C" espeak_ERROR espeak_SetVoiceByProperties(espeak_VOICE *voice_selector
 }  // end of espeak_SetVoiceByProperties
 
 
-extern "C" int espeak_GetParameter(espeak_PARAMETER parameter, int current)
+ESPEAK_API int espeak_GetParameter(espeak_PARAMETER parameter, int current)
 {//========================================================================
-  ENTER("espeak_GetParameter");
-  // current: 0=default value, 1=current value
-  if(current)
-    {
-      return(param_stack[0].parameter[parameter]);
-    }
-  else
-    {
-      return(param_defaults[parameter]);
-    }
+	ENTER("espeak_GetParameter");
+	// current: 0=default value, 1=current value
+	if(current)
+	{
+		return(param_stack[0].parameter[parameter]);
+	}
+	else
+	{
+		return(param_defaults[parameter]);
+	}
 }  //  end of espeak_GetParameter
 
 
-extern "C" espeak_ERROR espeak_SetParameter(espeak_PARAMETER parameter, int value, int relative)
+ESPEAK_API espeak_ERROR espeak_SetParameter(espeak_PARAMETER parameter, int value, int relative)
 {//=============================================================================================
   ENTER("espeak_SetParameter");
 
 #ifdef USE_ASYNC
 	espeak_ERROR a_error;
 
-	if(my_mode == AUDIO_OUTPUT_SYNCHRONOUS)
+	if(synchronous_mode)
 	{
 		SetParameter(parameter,value,relative);
 		return(EE_OK);
@@ -889,7 +1001,7 @@ extern "C" espeak_ERROR espeak_SetParameter(espeak_PARAMETER parameter, int valu
 }
 
 
-extern "C" espeak_ERROR espeak_SetPunctuationList(const wchar_t *punctlist)
+ESPEAK_API espeak_ERROR espeak_SetPunctuationList(const wchar_t *punctlist)
 {//================================================================
   ENTER("espeak_SetPunctuationList");
   // Set the list of punctuation which are spoken for "some".
@@ -897,7 +1009,7 @@ extern "C" espeak_ERROR espeak_SetPunctuationList(const wchar_t *punctlist)
 #ifdef USE_ASYNC
 	espeak_ERROR a_error;
 
-	if(my_mode == AUDIO_OUTPUT_SYNCHRONOUS)
+	if(synchronous_mode)
 	{
 		sync_espeak_SetPunctuationList(punctlist);
 		return(EE_OK);
@@ -917,30 +1029,30 @@ extern "C" espeak_ERROR espeak_SetPunctuationList(const wchar_t *punctlist)
 }  //  end of espeak_SetPunctuationList
 
 
-extern "C" void espeak_SetPhonemeTrace(int value, FILE *stream)
+ESPEAK_API void espeak_SetPhonemeTrace(int value, FILE *stream)
 {//============================================================
-  ENTER("espeak_SetPhonemes");
-  /* Controls the output of phoneme symbols for the text
-     value=0  No phoneme output (default)
-     value=1  Output the translated phoneme symbols for the text
-     value=2  as (1), but also output a trace of how the translation was done (matching rules and list entries)
-  */
-  option_phonemes = value;
-  f_trans = stream;
-  if(stream == NULL)
-    f_trans = stdout;
-
+	ENTER("espeak_SetPhonemes");
+	/* Controls the output of phoneme symbols for the text
+		value=0  No phoneme output (default)
+		value=1  Output the translated phoneme symbols for the text
+		value=2  as (1), but also output a trace of how the translation was done (matching rules and list entries)
+	*/
+	option_phonemes = value;
+	f_trans = stream;
+	if(stream == NULL)
+		f_trans = stdout;
+	
 }   //  end of espeak_SetPhonemes
 
 
-extern "C" void espeak_CompileDictionary(const char *path, FILE *log)
-{//==================================================================
-  ENTER("espeak_CompileDictionary");
-  CompileDictionary(path,dictionary_name,log,NULL);
+ESPEAK_API void espeak_CompileDictionary(const char *path, FILE *log, int flags)
+{//=============================================================================
+	ENTER("espeak_CompileDictionary");
+	CompileDictionary(path, dictionary_name, log, NULL, flags);
 }   //  end of espeak_CompileDirectory
 
 
-extern "C" espeak_ERROR espeak_Cancel(void)
+ESPEAK_API espeak_ERROR espeak_Cancel(void)
 {//===============================
 #ifdef USE_ASYNC
 	ENTER("espeak_Cancel");
@@ -957,7 +1069,7 @@ extern "C" espeak_ERROR espeak_Cancel(void)
 }   //  end of espeak_Cancel
 
 
-extern "C" int espeak_IsPlaying(void)
+ESPEAK_API int espeak_IsPlaying(void)
 {//==================================
 //	ENTER("espeak_IsPlaying");
 #ifdef USE_ASYNC
@@ -971,7 +1083,7 @@ extern "C" int espeak_IsPlaying(void)
 }   //  end of espeak_IsPlaying
 
 
-extern "C" espeak_ERROR espeak_Synchronize(void)
+ESPEAK_API espeak_ERROR espeak_Synchronize(void)
 {//=============================================
 #ifdef USE_ASYNC
 	SHOW_TIME("espeak_Synchronize > ENTER");
@@ -987,7 +1099,7 @@ extern "C" espeak_ERROR espeak_Synchronize(void)
 
 extern void FreePhData(void);
 
-extern "C" espeak_ERROR espeak_Terminate(void)
+ESPEAK_API espeak_ERROR espeak_Terminate(void)
 {//===========================================
 	ENTER("espeak_Terminate");
 #ifdef USE_ASYNC
@@ -1001,15 +1113,17 @@ extern "C" espeak_ERROR espeak_Terminate(void)
 		wave_terminate();
 	}
 
-	Free(event_list);
 #endif
+	Free(event_list);
+	event_list = NULL;
 	Free(outbuf);
+	outbuf = NULL;
 	FreePhData();
 
 	return EE_OK;
 }   //  end of espeak_Terminate
 
-extern "C" const char *espeak_Info(void *)
+ESPEAK_API const char *espeak_Info(void *)
 {//=======================================
 	return(version_string);
 }
