@@ -1,10 +1,10 @@
 /***************************************************************************
- *   Copyright (C) 2005,2006 by Jonathan Duddington                        *
- *   jonsd@users.sourceforge.net                                           *
+ *   Copyright (C) 2005 to 2007 by Jonathan Duddington                     *
+ *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation; either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
@@ -13,24 +13,38 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   along with this program; if not, write see:                           *
+ *               <http://www.gnu.org/licenses/>.                           *
  ***************************************************************************/
 
+
+#include <sys/types.h>
+
 // conditional compilation options
 
+#if defined(BYTE_ORDER) && BYTE_ORDER == BIG_ENDIAN
+#define ARCH_BIG
+#endif
 
 #define PLATFORM_POSIX
 #define PATHSEP  '/'
-#define USE_PORTAUDIO
+// USE_PORTAUDIO or USE_PULSEAUDIO are now defined in the makefile
+//#define USE_PORTAUDIO
+//#define USE_PULSEAUDIO
 #define USE_NANOSLEEP
 #define __cdecl 
+#define ESPEAK_API  extern "C"
 
 #ifdef LIBRARY
 #define USE_ASYNC
+//#define USE_MBROLA_LIB
 #endif
 
+#ifdef _ESPEAKEDIT
+#define USE_PORTAUDIO
+#define USE_ASYNC
+#define LOG_FRAMES      // write keyframe info to log-espeakedit
+#endif
 
 // will look for espeak_data directory here, and also in user's home directory
 #ifndef PATH_ESPEAK_DATA
@@ -43,9 +57,6 @@ typedef double DOUBLEX;
 
 
 
-#define N_PEAKS   9
-#define N_MARKERS 7
-
 
 typedef struct {
    const char *mnem;
@@ -54,31 +65,16 @@ typedef struct {
 int LookupMnem(MNEM_TAB *table, char *string);
 
 
-typedef struct {
-   short pkfreq;
-   short pkheight;
-   short pkwidth;
-   short pkright;
-}  peak_t;
-
-typedef struct {
-	short frflags;
-	unsigned char length;
-	unsigned char rms;
-	short ffreq[9];
-	unsigned char fheight[9];
-	unsigned char fwidth[6];          // width/4
-	unsigned char fright[6];          // width/4
-} frame_t;
+#ifdef PLATFORM_WINDOWS
+#define N_PATH_HOME  220
+#else
+#define N_PATH_HOME  150
+#endif
 
+extern char path_home[N_PATH_HOME];    // this is the espeak-data directory
 
+extern void strncpy0(char *to,const char *from, int size);
 int  GetFileLength(const char *filename);
 char *Alloc(int size);
 void Free(void *ptr);
 
-
-extern void strncpy0(char *to,const char *from, int size);
-extern char path_home[];
-extern const char *version_string;
-extern const int version_phdata;
-
