@@ -1,10 +1,10 @@
 /***************************************************************************
- *   Copyright (C) 2005,2006 by Jonathan Duddington                        *
- *   jsd@clara.co.uk                                                       *
+ *   Copyright (C) 2005 to 2007 by Jonathan Duddington                     *
+ *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation; either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
@@ -13,12 +13,12 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   along with this program; if not, write see:                           *
+ *               <http://www.gnu.org/licenses/>.                           *
  ***************************************************************************/
 
-#define N_PHONEME_LIST  700    // enough for source[] full of text, else it will truncate
+
+#define N_PHONEME_LIST  1000    // enough for source[N_TR_SOURCE] full of text, else it will truncate
 
 #define MAX_HARMONIC  400           // 400 * 50Hz = 20 kHz, more than enough
 #define N_SEQ_FRAMES   25           // max frames in a spectrum sequence (real max is ablut 8)
@@ -31,15 +31,19 @@
 #define FRFLAG_LEN_MOD         0x04   // reduce effect of length adjustment
 #define FRFLAG_BREAK_LF        0x08   // but keep f3 upwards
 #define FRFLAG_BREAK           0x10   // don't merge with next frame
+#define FRFLAG_BREAK_2         0x18   // FRFLAG_BREAK_LF or FRFLAG_BREAK
 #define FRFLAG_FORMANT_RATE    0x20   // Flag5 allow increased rate of change of formant freq
 #define FRFLAG_MODULATE        0x40   // Flag6 modulate amplitude of some cycles to give trill
+#define FRFLAG_DEFER_WAV       0x80   // Flag7 defer mixing WAV until the next frame
+#define FRFLAG_COPIED        0x8000   // This frame has been copied into temporary rw memory
 
-#define SFLAG_SEQCONTINUE      0x01
+#define SFLAG_SEQCONTINUE      0x01   // a liquid or nasal after a vowel, but not followed by a vowel
 #define SFLAG_EMBEDDED         0x02   // there are embedded commands before this phoneme
 #define SFLAG_SYLLABLE         0x04   // vowel or syllabic consonant
 #define SFLAG_LENGTHEN         0x08   // lengthen symbol : included after this phoneme
 #define SFLAG_DICTIONARY       0x10   // the pronunciation of this word was listed in the xx_list dictionary
-
+#define SFLAG_SWITCHED_LANG    0x20   // this word uses phonemes from a different language
+#define SFLAG_PROMOTE_STRESS   0x40   // this unstressed word can be promoted to stressed
 
 // embedded command numbers
 #define EMBED_P     1   // pitch
@@ -47,7 +51,7 @@
 #define EMBED_A     3   // amplitude/volume
 #define EMBED_R     4   // pitch range/expression
 #define EMBED_H     5   // echo/reverberation
-#define EMBED_T     6   // different tone
+#define EMBED_T     6   // different tone for announcing punctuation
 #define EMBED_I     7   // sound icon
 #define EMBED_S2    8   // speed (used in synthesize)
 #define EMBED_Y     9   // say-as commands
@@ -61,6 +65,28 @@ extern int embedded_value[N_EMBEDDED_VALUES];
 extern int embedded_default[N_EMBEDDED_VALUES];
 
 
+#define N_PEAKS   9
+#define N_MARKERS 8
+
+typedef struct {
+   short pkfreq;
+   short pkheight;
+   short pkwidth;
+   short pkright;
+}  peak_t;
+
+typedef struct {
+	short frflags;
+	unsigned char length;
+	unsigned char rms;
+	short ffreq[9];
+	unsigned char fheight[9];
+	unsigned char fwidth[6];          // width/4
+	unsigned char fright[6];          // width/4
+} frame_t;
+
+
+
 // formant data used by wavegen
 typedef struct {
 	int freq;     // Hz<<16
@@ -77,6 +103,15 @@ typedef struct {
 	DOUBLEX right_inc;
 }  wavegen_peaks_t;
 
+typedef struct {
+	double a;
+	double b;
+	double c;
+	double x1;
+	double x2;
+}  RESONATOR;
+
+
 typedef struct {
    short length;
    unsigned char  n_frames;
@@ -86,7 +121,7 @@ typedef struct {
 
 typedef struct {
 	short length;
-	short flags;
+	short frflags;
 	frame_t *frame;
 } frameref_t;
 
@@ -115,6 +150,16 @@ typedef struct {
 	char *filename;
 } SOUND_ICON;
 
+typedef struct {
+	int  name;
+	unsigned int  next_phoneme;
+	int  mbr_name;
+	int  mbr_name2;
+	int  percent;         // percentage length of first component
+	int  control;
+} MBROLA_TAB;
+
+
 // phoneme table
 extern PHONEME_TAB *phoneme_tab[N_PHONEME_TAB];
 
@@ -127,6 +172,8 @@ extern unsigned char env_fall[128];
 extern unsigned char env_rise[128];
 extern unsigned char env_frise[128];
 
+#define MAX_PITCH_VALUE  101
+extern unsigned char pitch_adjust_tab[MAX_PITCH_VALUE+1];
 
 // queue of commands for wavegen
 #define WCMD_AMPLITUDE 1
@@ -141,7 +188,7 @@ extern unsigned char env_frise[128];
 #define WCMD_EMBEDDED 10
 
 
-#define N_WCMDQ   200
+#define N_WCMDQ   160
 #define MIN_WCMDQ  20   // need this many free entries before adding new phoneme
 
 extern long wcmdq[N_WCMDQ][4];
@@ -157,24 +204,24 @@ int  WavegenOpenSound();
 int  WavegenCloseSound();
 int  WavegenInitSound();
 void WavegenInit(int rate, int wavemult_fact);
-int  OpenWaveFile(const char *path, int rate);
-void CloseWaveFile(int rate);
 float polint(float xa[],float ya[],int n,float x);
-int  WavegenFile(void);
 int WavegenFill(int fill_zeros);
 void MarkerEvent(int type, unsigned int char_position, int value, unsigned char *out_ptr);
-extern unsigned char *out_ptr;
-extern unsigned char *out_end;
 
 
 extern unsigned char *wavefile_data;
 extern int samplerate;
+extern int samplerate_native;
 
 extern int wavefile_ix;
 extern int wavefile_amp;
+extern int wavefile_ix2;
+extern int wavefile_amp2;
 extern int vowel_transition[4];
 extern int vowel_transition0, vowel_transition1;
 
+extern char mbrola_name[20];
+
 // from synthdata file
 unsigned int LookupSound(PHONEME_TAB *ph1, PHONEME_TAB *ph2, int which, int *match_level, int control);
 frameref_t *LookupSpect(PHONEME_TAB *ph1, PHONEME_TAB *prev_ph, PHONEME_TAB *next_ph, int which, int *match_level, int *n_frames, PHONEME_LIST *plist);
@@ -183,7 +230,7 @@ unsigned char *LookupEnvelope(int ix);
 int LoadPhData();
 
 void SynthesizeInit(void);
-int  Generate(PHONEME_LIST *phoneme_list, int n_phoneme_list, int resume);
+int  Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume);
 void MakeWave2(PHONEME_LIST *p, int n_ph);
 int  SynthOnTimer(void);
 int  SpeakNextClause(FILE *f_text, const void *text_in, int control);
@@ -193,9 +240,40 @@ void SetEmbedded(int control, int value);
 void SelectPhonemeTable(int number);
 int  SelectPhonemeTableName(const char *name);
 
-extern unsigned char *envelope_data[16];
-extern int formant_rate[];         // max rate of change of each formant
 
-#define N_SOUNDICON_TAB  100
+extern unsigned char *envelope_data[18];
+extern int formant_rate[];         // max rate of change of each formant
+extern int speed_factor1;
+extern int speed_factor2;
+extern int speed_min_sample_len;
+
+extern long count_samples;
+extern int outbuf_size;
+extern unsigned char *out_ptr;
+extern unsigned char *out_start;
+extern unsigned char *out_end;
+extern int event_list_ix;
+extern espeak_EVENT *event_list;
+extern t_espeak_callback* synth_callback;
+extern int option_log_frames;
+extern const char *version_string;
+extern const int version_phdata;
+
+#define N_SOUNDICON_TAB  80   // total entries in soundicon_tab
+#define N_SOUNDICON_SLOTS 4    // number of slots reserved for dynamic loading of autio files
 extern int n_soundicon_tab;
 extern SOUND_ICON soundicon_tab[N_SOUNDICON_TAB];
+
+espeak_ERROR SetVoiceByName(const char *name);
+espeak_ERROR SetVoiceByProperties(espeak_VOICE *voice_selector);
+espeak_ERROR LoadMbrolaTable(const char *mbrola_voice, const char *phtrans, int srate);
+void SetParameter(int parameter, int value, int relative);
+void MbrolaTranslate(PHONEME_LIST *plist, int n_phonemes, FILE *f_mbrola);
+int MbrolaSynth(char *p_mbrola);
+int DoSample(PHONEME_TAB *ph1, PHONEME_TAB *ph2, int which, int length_mod, int amp);
+int DoSpect(PHONEME_TAB *this_ph, PHONEME_TAB *prev_ph, PHONEME_TAB *next_ph,
+		int which, PHONEME_LIST *plist, int modulation);
+int PauseLength(int pause);
+int LookupPhonemeTable(const char *name);
+
+void InitBreath(void);
