@@ -29,7 +29,9 @@
 #ifdef PLATFORM_WINDOWS
 #include "windows.h"
 #else
-#ifndef PLATFORM_RISCOS
+#ifdef PLATFORM_RISCOS
+#include "kernel.h"
+#else
 #include "dirent.h"
 #endif
 #endif
@@ -64,8 +66,7 @@ static int n_voices_list = 0;
 static espeak_VOICE *voices_list[N_VOICES_LIST];
 static int len_path_voices;
 
-espeak_VOICE *voice_selected = NULL;
-espeak_VOICE *first_voice = NULL;
+espeak_VOICE voice_selected;
 
 
 
@@ -104,7 +105,7 @@ espeak_VOICE *first_voice = NULL;
 
 // these need a phoneme table to have been specified
 #define V_REPLACE    28
-
+#define V_CONSONANTS 29
 
 
 typedef struct {
@@ -142,6 +143,7 @@ static keywtab_t keyword_tab[] = {
 	{"numbers",    V_NUMBERS},
 	{"option",     V_OPTION},
 	{"mbrola",     V_MBROLA},
+	{"consonants", V_CONSONANTS},
 
 	// these just set a value in langopts.param[]
 	{"l_dieresis", 0x100+LOPT_DIERESES},
@@ -153,16 +155,33 @@ static keywtab_t keyword_tab[] = {
 	{"l_length_mods", 0x100+LOPT_LENGTH_MODS},
 	{NULL,   0} };
 
-#define N_VOICES 100
-static int n_voices_tab = 0;
-static voice_t *voices_tab[N_VOICES];
-
 #define N_VOICE_VARIANTS   12
 const char variants_either[N_VOICE_VARIANTS] = {1,2,12,3,13,4,14,5,11,0};
 const char variants_male[N_VOICE_VARIANTS] = {1,2,3,4,5,0};
 const char variants_female[N_VOICE_VARIANTS] = {11,12,13,14,0};
 const char *variant_lists[3] = {variants_either, variants_male, variants_female};
 
+voice_t voicedata;
+voice_t *voice = &voicedata;
+
+char *fgets_strip(char *buf, int size, FILE *f_in)
+{//===============================================
+// strip trailing spaces, and truncate lines at // comment
+	int len;
+	char *p;
+
+	if(fgets(buf,size,f_in) == NULL)
+		return(NULL);
+
+	len = strlen(buf);
+	while((--len > 0) && isspace(buf[len]))
+		buf[len] = 0;
+
+	if((p = strstr(buf,"//")) != NULL)
+		*p = 0;
+
+	return(buf);
+}
 
 
 void SetToneAdjust(voice_t *voice, int *tone_pts)
@@ -225,7 +244,7 @@ static espeak_VOICE *ReadVoiceFile(FILE *f_in, const char *fname, const char*lea
 // Read a Voice file, allocate a VOICE_DATA and set data from the
 // file's  language, gender, name  lines
 
-	char linebuf[80];
+	char linebuf[120];
 	char vname[80];
 	char vgender[80];
 	char vlanguage[80];
@@ -260,10 +279,8 @@ static espeak_VOICE *ReadVoiceFile(FILE *f_in, const char *fname, const char*lea
 	vgender[0] = 0;
 	age = 0;
 
-	while(fgets(linebuf,sizeof(linebuf),f_in) != NULL)
+	while(fgets_strip(linebuf,sizeof(linebuf),f_in) != NULL)
 	{
-		linebuf[strlen(linebuf)-1] = 0;
-
 		if(memcmp(linebuf,"name",4)==0)
 		{
 			p = &linebuf[4];
@@ -354,6 +371,9 @@ void VoiceReset(int tone_only)
 	voice->n_harmonic_peaks = 5;
 	voice->peak_shape = 1;
 	voice->voicing = 64;
+	voice->consonant_amp = 100;
+	voice->consonant_ampv = 100;
+
 #ifdef PLATFORM_RISCOS
 	voice->roughness = 1;
 #else
@@ -368,10 +388,12 @@ void VoiceReset(int tone_only)
 		voice->width[pk] = 256;
 		voice->breath[pk] = 0;
 		voice->breathw[pk] = breath_widths[pk];  // default breath formant woidths
+		voice->freqadd[pk] = 0;
 
 		// adjust formant smoothing depending on sample rate
 		formant_rate[pk] = (formant_rate_22050[pk] * 22050)/samplerate;
 	}
+	voice->height[2] = 240;  // reduce F2 slightly
 
 	// This table provides the opportunity for tone control.
 	// Adjustment of harmonic amplitudes, steps of 8Hz
@@ -389,7 +411,7 @@ SetToneAdjust(voice,tone_points);
 		n_replace_phonemes = 0;
 		option_tone1 = 0;
 		option_quiet = 0;
-		LoadMbrolaTable(NULL,NULL);
+		LoadMbrolaTable(NULL,NULL,0);
 	}
 }  // end of VoiceReset
 
@@ -399,11 +421,12 @@ static void VoiceFormant(char *p)
 	// Set parameters for a formant
 	int ix;
 	int formant;
-	int freq = -1;
-	int height = -1;
-	int width = -1;
+	int freq = 100;
+	int height = 100;
+	int width = 100;
+	int freqadd = 0;
 
-	ix = sscanf(p,"%d %d %d %d",&formant,&freq,&height,&width);
+	ix = sscanf(p,"%d %d %d %d %d",&formant,&freq,&height,&width,&freqadd);
 	if(ix < 2)
 		return;
 
@@ -416,37 +439,10 @@ static void VoiceFormant(char *p)
 		voice->height[formant] = int(height * 2.56001);
 	if(width >= 0)
 		voice->width[formant] = int(width * 2.56001);
+	voice->freqadd[formant] = freqadd;
 }
 
 
-static voice_t *VoiceLookup(char *voicename)
-{//=========================================
-// Keep a cache of previously used voices.
-	int ix;
-	voice_t *v;
-
-	for(ix=0; ix < N_VOICES; ix++)
-	{
-		if((ix < n_voices_tab) && (strcmp(voices_tab[ix]->name,voicename)==0))
-		{
-			return(voices_tab[ix]);   // found the entry for the specified voice name
-		}
-
-		if(ix == n_voices_tab)
-		{
-			// found a free slot
-			v = (voice_t *)Alloc(sizeof(voice_t));
-			if(v == NULL)
-				return(NULL);
-			voices_tab[n_voices_tab++] = v;
-			strncpy0(v->name,voicename,sizeof(v->name));
-			return(v);
-		}
-	}
-	return(voices_tab[0]);  // table is full, reuse the first entry
-}  // end of VoiceLookup
-
-
 
 
 
@@ -482,7 +478,7 @@ static int Read8Numbers(char *data_in,int *data)
 
 
 voice_t *LoadVoice(const char *vname, int control)
-{//==========================================
+{//===============================================
 // control, bit 0  1= no_default
 //          bit 1  1 = change tone only, not language
 //          bit 2  1 = don't report error on LoadDictionary
@@ -496,6 +492,7 @@ voice_t *LoadVoice(const char *vname, int control)
 	int  n;
 	int  value;
 	int  error = 0;
+	int  langix = 0;
 	int  tone_only = control & 2;
 	int  language_set = 0;
 	int  phonemes_set = 0;
@@ -505,7 +502,6 @@ voice_t *LoadVoice(const char *vname, int control)
 	int  conditional_rules = 0;
 	LANGUAGE_OPTIONS *langopts = NULL;
 
-	voice_t *v;
 	Translator *new_translator = NULL;
 
 	char voicename[40];
@@ -513,7 +509,7 @@ voice_t *LoadVoice(const char *vname, int control)
 	char translator_name[40];
 	char new_dictionary[40];
 	char phonemes_name[40];
-	char *language_type;
+	const char *language_type;
 	char buf[200];
 	char path_voices[sizeof(path_home)+12];
 	char langname[4];
@@ -525,6 +521,10 @@ voice_t *LoadVoice(const char *vname, int control)
 	int pitch1;
 	int pitch2;
 
+	static char voice_identifier[40];  // file name for  voice_selected
+	static char voice_name[40];        // voice name for voice_selected
+	static char voice_languages[100];  // list of languages and priorities for voice_selected
+
 	strcpy(voicename,vname);
 	if(voicename[0]==0)
 		strcpy(voicename,"default");
@@ -562,13 +562,6 @@ voice_t *LoadVoice(const char *vname, int control)
 			language_type = voicename;
 	}
 
-	if((first_voice == NULL) && (f_voice != NULL))
-	{
-		first_voice = ReadVoiceFile(f_voice,buf+strlen(path_voices),voicename);
-		rewind(f_voice);
-	}
-
-
 	if(!tone_only && (translator != NULL))
 	{
 		delete translator;
@@ -582,8 +575,23 @@ voice_t *LoadVoice(const char *vname, int control)
 
 	if(!tone_only)
 	{
-		if((v = VoiceLookup(voicename)) != NULL)
-			voice = v;
+		voice = &voicedata;
+		strncpy0(voice_identifier,vname,sizeof(voice_identifier));
+		voice_name[0] = 0;
+		voice_languages[0] = 0;
+
+		voice_selected.identifier = voice_identifier;
+		voice_selected.name = voice_name;
+		voice_selected.languages = voice_languages;
+	}
+	else
+	{
+		// append the variant file name to the voice identifier
+		if((p = strchr(voice_identifier,'+')) != NULL)
+			*p = 0;    // remove previous variant name
+		sprintf(buf,"+%s",&vname[3]);    // omit  !v/  from the variant filename
+		strcat(voice_identifier,buf);
+		langopts = &translator->langopts;
 	}
 	VoiceReset(tone_only);
 
@@ -591,11 +599,8 @@ voice_t *LoadVoice(const char *vname, int control)
 		SelectPhonemeTableName(phonemes_name);  // set up phoneme_tab
 
 
-	while((f_voice != NULL) && (fgets(buf,sizeof(buf),f_voice) != NULL))
+	while((f_voice != NULL) && (fgets_strip(buf,sizeof(buf),f_voice) != NULL))
 	{
-		if((p = strstr(buf,"//")) != NULL)
-			*p = 0;
-
 		// isolate the attribute name
 		for(p=buf; (*p != 0) && !isspace(*p); p++);
 		*p++ = 0;
@@ -615,33 +620,68 @@ voice_t *LoadVoice(const char *vname, int control)
 		switch(key)
 		{
 		case V_LANGUAGE:
-			// only act on the first language line
-			if(language_set || tone_only)
-				break;
+			{
+				unsigned int len;
+				int priority;
 
-			sscanf(p,"%s",language_name);
-			if(strcmp(language_name,"variant")==0)
-				break;
-
-			language_type = strtok(language_name,"-");
-			language_set = 1;
-			strcpy(translator_name,language_type);
-			strcpy(new_dictionary,language_type);
-			strcpy(phonemes_name,language_type);
-			SelectPhonemeTableName(phonemes_name);
-
-			if(new_translator != NULL)
-					delete new_translator;
-
-			new_translator = SelectTranslator(translator_name);
-			langopts = &new_translator->langopts;
+				if(tone_only)
+					break;
+	
+				priority = DEFAULT_LANGUAGE_PRIORITY;
+				language_name[0] = 0;
+	
+				sscanf(p,"%s %d",language_name,&priority);
+				if(strcmp(language_name,"variant") == 0)
+					break;
+	
+				len = strlen(language_name) + 2;
+				// check for space in languages[]
+				if(len < (sizeof(voice_languages)-langix-1))
+				{
+					voice_languages[langix] = priority;
+	
+					strcpy(&voice_languages[langix+1],language_name);
+					langix += len;
+				}
+	
+				// only act on the first language line
+				if(language_set == 0)
+				{
+					language_type = strtok(language_name,"-");
+					language_set = 1;
+					strcpy(translator_name,language_type);
+					strcpy(new_dictionary,language_type);
+					strcpy(phonemes_name,language_type);
+					SelectPhonemeTableName(phonemes_name);
+		
+					if(new_translator != NULL)
+							delete new_translator;
+		
+					new_translator = SelectTranslator(translator_name);
+					langopts = &new_translator->langopts;
+				}
+			}
 			break;
 
 		case V_NAME:
-		case V_GENDER:
+			if(tone_only == 0)
+			{
+				while(isspace(*p)) p++;
+				strncpy0(voice_name,p,sizeof(voice_name));
+			}
 			break;
 
-		case V_TRANSLATOR:        // language_name
+		case V_GENDER:
+			{
+				int age;
+				char vgender[80];
+				sscanf(p,"%s %d",vgender,&age);
+				voice_selected.gender = LookupMnem(genders,vgender);
+				voice_selected.age = age;
+			}
+			break;
+
+		case V_TRANSLATOR:
 			if(tone_only) break;
 
 			sscanf(p,"%s",translator_name);
@@ -690,7 +730,9 @@ voice_t *LoadVoice(const char *vname, int control)
 			break;
 
 		case V_INTONATION:   // intonation
-			sscanf(p,"%d %d",&option_tone1,&option_tone2);
+			sscanf(p,"%d %d",&option_tone_flags,&option_tone2);
+			if((option_tone_flags & 0xff) != 0)
+				langopts->intonation_group = option_tone_flags & 0xff;
 			break;
 
 		case V_DICTRULES:   // conditional dictionary rules and list entries
@@ -698,7 +740,7 @@ voice_t *LoadVoice(const char *vname, int control)
 			{
 				while(isspace(*p)) p++;
 				n = -1;
-				if((n = atoi(p)) > 0)
+				if(((n = atoi(p)) > 0) && (n < 32))
 				{
 					p++;
 					conditional_rules |= (1 << n);
@@ -800,13 +842,19 @@ voice_t *LoadVoice(const char *vname, int control)
 				voice->breathw[0] = Read8Numbers(p,&voice->breathw[1]);
 			break;
 
+		case V_CONSONANTS:
+			value = sscanf(p,"%d %d",&voice->consonant_amp, &voice->consonant_ampv);
+			break;
+
 		case V_MBROLA:
 			{
+				int srate = 16000;
 				char name[40];
 				char phtrans[40];
+
 				phtrans[0] = 0;
-				sscanf(p,"%s %s",name,phtrans);
-				LoadMbrolaTable(name,phtrans);
+				sscanf(p,"%s %s %d",name,phtrans,&srate);
+				LoadMbrolaTable(name,phtrans,srate);
 			}
 			break;
 
@@ -854,6 +902,8 @@ voice_t *LoadVoice(const char *vname, int control)
 			return(NULL);   // no dictionary loaded
 
 		new_translator->dict_condition = conditional_rules;
+
+		voice_languages[langix] = 0;
 	}
 
 	langopts = &new_translator->langopts;
@@ -897,31 +947,36 @@ char *ExtractVoiceVariantName(char *vname, int variant_num)
 
 	char *p;
 	static char variant_name[20];
+	char variant_prefix[5];
 
 	variant_name[0] = 0;
+	sprintf(variant_prefix,"!v%c",PATHSEP);
 
-	if((p = strchr(vname,'+')) != NULL)
+	if(vname != NULL)
 	{
-		// The voice name has a +variant suffix
-		*p++ = 0;   // delete the suffix from the voice name
-		if(isdigit(*p))
+		if((p = strchr(vname,'+')) != NULL)
 		{
-			variant_num = atoi(p);  // variant number
+			// The voice name has a +variant suffix
+			*p++ = 0;   // delete the suffix from the voice name
+			if(isdigit(*p))
+			{
+				variant_num = atoi(p);  // variant number
+			}
+			else
+			{
+				// voice variant name, not number
+				strcpy(variant_name,variant_prefix);
+				strncpy0(&variant_name[3],p,sizeof(variant_name)-3);
+			}	
 		}
-		else
-		{
-			// voice variant name, not number
-			strcpy(variant_name,"!v/");
-			strncpy0(&variant_name[3],p,sizeof(variant_name)-3);
-		}	
 	}
 	
 	if(variant_num > 0)
 	{
 		if(variant_num < 10)
-			sprintf(variant_name,"!v/m%d",variant_num);  // male
+			sprintf(variant_name,"%sm%d",variant_prefix, variant_num);  // male
 		else
-			sprintf(variant_name,"!v/f%d",variant_num-10);  // female
+			sprintf(variant_name,"%sf%d",variant_prefix, variant_num-10);  // female
 	}
 
 	return(variant_name);
@@ -980,10 +1035,10 @@ static int __cdecl VoiceScoreSorter(const void *p1, const void *p2)
 }
 
 
-static int ScoreVoice(espeak_VOICE *voice_spec, int spec_n_parts, int spec_lang_len, espeak_VOICE *voice)
-{//======================================================================================================
+static int ScoreVoice(espeak_VOICE *voice_spec, const char *spec_language, int spec_n_parts, int spec_lang_len, espeak_VOICE *voice)
+{//=========================================================================================================================
 	int ix;
-	char *p;
+	const char *p;
 	int c1, c2;
 	int language_priority;
 	int n_parts;
@@ -997,13 +1052,21 @@ static int ScoreVoice(espeak_VOICE *voice_spec, int spec_n_parts, int spec_lang_
 
 	p = voice->languages;  // list of languages+dialects for which this voice is suitable
 
+	if(strcmp(spec_language,"mbrola")==0)
+	{
+		// only list mbrola voices
+		if(memcmp(voice->identifier,"mb/",3) == 0)
+			return(100);
+		return(0);
+	}
+
 	if(spec_n_parts == 0)
 	{
 		score = 100;
 	}
 	else
 	{
-		if((*p == 0) && (strcmp(voice_spec->languages,"variants")==0))
+		if((*p == 0) && (strcmp(spec_language,"variants")==0))
 		{
 			// match on a voice with no languages if the required language is "variants"
 			score = 100;
@@ -1020,7 +1083,7 @@ static int ScoreVoice(espeak_VOICE *voice_spec, int spec_n_parts, int spec_lang_
 
 			for(ix=0; ; ix++)
 			{
-				if((ix >= spec_lang_len) || ((c1 = voice_spec->languages[ix]) == '-'))
+				if((ix >= spec_lang_len) || ((c1 = spec_language[ix]) == '-'))
 					c1 = 0;
 				if((c2 = p[ix]) == '-')
 					c2 = 0;
@@ -1124,17 +1187,17 @@ static int SetVoiceScores(espeak_VOICE *voice_select, espeak_VOICE **voices, int
 	int nv;           // number of candidates
 	int n_parts=0;
 	int lang_len=0;
-	char *p;
 	espeak_VOICE *vp;
+	char language[80];
 
 	// count number of parts in the specified language
 	if((voice_select->languages != NULL) && (voice_select->languages[0] != 0))
 	{
 		n_parts = 1;
 		lang_len = strlen(voice_select->languages);
-		for(p = voice_select->languages; *p != 0; p++)
+		for(ix=0; (ix<=lang_len) && ((unsigned)ix < sizeof(language)); ix++)
 		{
-			if(*p == '-')
+			if((language[ix] = tolower(voice_select->languages[ix])) == '-')
 				n_parts++;
 		}
 	}
@@ -1143,10 +1206,11 @@ static int SetVoiceScores(espeak_VOICE *voice_select, espeak_VOICE **voices, int
 	for(ix=0; ix<n_voices_list; ix++)
 	{
 		vp = voices_list[ix];
+
 		if(((control & 1) == 0) && (memcmp(vp->identifier,"mb/",3) == 0))
 			continue;
 
-		if((score = ScoreVoice(voice_select, n_parts, lang_len, voices_list[ix])) > 0)
+		if((score = ScoreVoice(voice_select, language, n_parts, lang_len, voices_list[ix])) > 0)
 		{
 			voices[nv++] = vp;
 			vp->score = score;
@@ -1166,16 +1230,23 @@ static int SetVoiceScores(espeak_VOICE *voice_select, espeak_VOICE **voices, int
 
 
 
-static espeak_VOICE *SelectVoiceByName(espeak_VOICE **voices, const char *name)
-{//============================================================================
+espeak_VOICE *SelectVoiceByName(espeak_VOICE **voices, const char *name)
+{//=====================================================================
 	int ix;
 	int match_fname = -1;
 	int match_fname2 = -1;
 	int match_name = -1;
-	char *id;
+	const char *id;
 	int last_part_len;
 	char last_part[41];
 
+	if(voices == NULL)
+	{
+		if(n_voices_list == 0)
+			espeak_ListVoices(NULL);   // create the voices list
+		voices = voices_list;
+	}
+
 	sprintf(last_part,"%c%s",PATHSEP,name);
 	last_part_len = strlen(last_part);
 
@@ -1214,8 +1285,8 @@ static espeak_VOICE *SelectVoiceByName(espeak_VOICE **voices, const char *name)
 
 
 
-espeak_VOICE *SelectVoice(espeak_VOICE *voice_select, int *variant)
-{//================================================================
+char const *SelectVoice(espeak_VOICE *voice_select)
+{//================================================
 // Returns a path within espeak-voices, with a possible +variant suffix
 // variant is an output-only parameter
 	int nv;           // number of candidates
@@ -1226,6 +1297,7 @@ espeak_VOICE *SelectVoice(espeak_VOICE *voice_select, int *variant)
 	int gender;
 	int skip;
 	int aged=1;
+	char *variant_name;
 	const char *p, *p_start;
 	espeak_VOICE *vp = NULL;
 	espeak_VOICE *vp2;
@@ -1233,6 +1305,7 @@ espeak_VOICE *SelectVoice(espeak_VOICE *voice_select, int *variant)
 	espeak_VOICE *voices[N_VOICES_LIST]; // list of candidates
 	espeak_VOICE *voices2[N_VOICES_LIST+N_VOICE_VARIANTS];
 	static espeak_VOICE voice_variants[N_VOICE_VARIANTS];
+	static char voice_id[50];
 
 	memcpy(&voice_select2,voice_select,sizeof(voice_select2));
 
@@ -1242,8 +1315,6 @@ espeak_VOICE *SelectVoice(espeak_VOICE *voice_select, int *variant)
 	if((voice_select2.languages == NULL) || (voice_select2.languages[0] == 0))
 	{
 		// no language is specified. Get language from the named voice
-		int var;
-		char *p2;
 		static char buf[60];
 	
 		if(voice_select2.name == NULL)
@@ -1253,12 +1324,8 @@ espeak_VOICE *SelectVoice(espeak_VOICE *voice_select, int *variant)
 		}
 	
 		strncpy0(buf,voice_select2.name,sizeof(buf));
-		if((p2 = strchr(buf,'+')) != NULL)
-		{
-			// remove the voice variant suffix, from eg. en+3
-			*p2 = 0;
-			var = atoi(p2+1);
-		}
+		variant_name = ExtractVoiceVariantName(buf,0);
+
 		vp = SelectVoiceByName(voices_list,buf);
 		if(vp != NULL)
 		{
@@ -1266,8 +1333,13 @@ espeak_VOICE *SelectVoice(espeak_VOICE *voice_select, int *variant)
 
 			if((voice_select2.gender==0) && (voice_select2.age==0) && (voice_select2.variant==0))
 			{
-				*variant = var;
-				return(vp);
+				if(variant_name[0] != 0)
+				{
+					sprintf(voice_id,"%s+%s",vp->identifier,&variant_name[3]);  // omit the  !v/  from variant_name
+					return(voice_id);
+				}
+
+				return(vp->identifier);
 			}
 		}
 	}
@@ -1344,8 +1416,15 @@ espeak_VOICE *SelectVoice(espeak_VOICE *voice_select, int *variant)
 
 	// index the sorted list by the required variant number
 	vp = voices2[voice_select2.variant % ix2];
-	*variant = vp->variant;
-	return(vp);
+
+	if(vp->variant != 0)
+	{
+		variant_name = ExtractVoiceVariantName(NULL,vp->variant);
+		sprintf(voice_id,"%s+%s",vp->identifier,&variant_name[3]);
+		return(voice_id);
+	}
+
+	return(vp->identifier);
 }  //  end of SelectVoice
 
 
@@ -1358,6 +1437,54 @@ void GetVoices(const char *path)
 	char fname[sizeof(path_home)+100];
 
 #ifdef PLATFORM_RISCOS
+	int len;
+	int *type;
+	char *p;
+	_kernel_swi_regs regs;
+	_kernel_oserror *error;
+	char buf[80];
+	char directory2[sizeof(path_home)+100];
+
+	regs.r[0] = 10;
+	regs.r[1] = (int)path;
+	regs.r[2] = (int)buf;
+	regs.r[3] = 1;
+	regs.r[4] = 0;
+	regs.r[5] = sizeof(buf);
+	regs.r[6] = 0;
+
+	while(regs.r[3] > 0)
+	{
+		error = _kernel_swi(0x0c+0x20000,&regs,&regs);      /* OS_GBPB 10, read directory entries */
+		if((error != NULL) || (regs.r[3] == 0))
+		{
+			break;
+		}
+		type = (int *)(&buf[16]);
+		len = strlen(&buf[20]);
+		sprintf(fname,"%s.%s",path,&buf[20]);
+
+		if(*type == 2)
+		{
+			// a sub-directory
+			GetVoices(fname);
+		}
+		else
+		{
+			// a regular line, add it to the voices list	
+			if((f_voice = fopen(fname,"r")) == NULL)
+				continue;
+		
+			// pass voice file name within the voices directory
+			voice_data = ReadVoiceFile(f_voice, fname+len_path_voices, &buf[20]);
+			fclose(f_voice);
+
+			if(voice_data != NULL)
+			{
+				voices_list[n_voices_list++] = voice_data;
+			}
+		}
+	}
 #else
 #ifdef PLATFORM_WINDOWS
    WIN32_FIND_DATA FindFileData;
@@ -1455,21 +1582,20 @@ espeak_ERROR SetVoiceByName(const char *name)
 	variant_name = ExtractVoiceVariantName(buf,0);
 
 	memset(&voice_selector,0,sizeof(voice_selector));
-	voice_selector.name = buf;
+//	voice_selector.name = buf;
+	voice_selector.name = name;  // include variant name in voice stack ??
 
 	// first check for a voice with this filename
 	// This may avoid the need to call espeak_ListVoices().
 
-	if((first_voice == NULL) && (LoadVoice(buf,1) != NULL))
+	if(LoadVoice(buf,1) != NULL)
 	{
-		voice_selected = first_voice;
-
 		if(variant_name[0] != 0)
 		{
 			LoadVoice(variant_name,2);
 		}
 
-		WavegenSetVoice(voice);
+		DoVoiceChange(voice);
 		SetVoiceStack(&voice_selector);
 		return(EE_OK);
 	}
@@ -1481,13 +1607,11 @@ espeak_ERROR SetVoiceByName(const char *name)
 	{
 		if(LoadVoice(v->identifier,0) != NULL)
 		{
-			voice_selected = v;
-
 			if(variant_name[0] != 0)
 			{
 				LoadVoice(variant_name,2);
 			}
-			WavegenSetVoice(voice);
+			DoVoiceChange(voice);
 			SetVoiceStack(&voice_selector);
 			return(EE_OK);
 		}
@@ -1499,12 +1623,12 @@ espeak_ERROR SetVoiceByName(const char *name)
 
 espeak_ERROR SetVoiceByProperties(espeak_VOICE *voice_selector)
 {//============================================================
-	int variant;
+	const char *voice_id;
 
-	voice_selected = SelectVoice(voice_selector,&variant);
+	voice_id = SelectVoice(voice_selector);
 
-	LoadVoiceVariant(voice_selected->identifier,variant);
-	WavegenSetVoice(voice);
+	LoadVoiceVariant(voice_id,0);
+	DoVoiceChange(voice);
 	SetVoiceStack(voice_selector);
 
 	return(EE_OK);
@@ -1526,15 +1650,9 @@ ESPEAK_API const espeak_VOICE **espeak_ListVoices(espeak_VOICE *voice_spec)
 	int j;
 	espeak_VOICE *v;
 	static espeak_VOICE *voices[N_VOICES_LIST];
-	char selected_voice_id[80];
 	char path_voices[sizeof(path_home)+12];
 
 	// free previous voice list data
-	if((voice_selected != NULL) && (voice_selected->identifier != NULL))
-		strncpy0(selected_voice_id,voice_selected->identifier,sizeof(selected_voice_id));
-	else
-		selected_voice_id[0] = 0;
-	voice_selected = NULL;
 
 	for(ix=0; ix<n_voices_list; ix++)
 	{
@@ -1553,18 +1671,6 @@ ESPEAK_API const espeak_VOICE **espeak_ListVoices(espeak_VOICE *voice_spec)
 	qsort(voices_list,n_voices_list,sizeof(espeak_VOICE *),
 		(int (__cdecl *)(const void *,const void *))VoiceNameSorter);
 
-	// restore pointer to current voice
-	if(selected_voice_id[0] != 0)
-	{
-		for(ix=0; ix<n_voices_list; ix++)
-		{
-			if(strcmp(selected_voice_id, voices_list[ix]->identifier)==0)
-			{
-				voice_selected = voices_list[ix];
-				break;
-			}
-		}
-	}
 
 	if(voice_spec)
 	{
@@ -1573,11 +1679,11 @@ ESPEAK_API const espeak_VOICE **espeak_ListVoices(espeak_VOICE *voice_spec)
 	}
 	else
 	{
-		// omit variant voices
+		// list all: omit variant voices and mbrola voices
 		j = 0;
 		for(ix=0; (v = voices_list[ix]) != NULL; ix++)
 		{
-			if((v->languages[0] != 0) && (strcmp(&v->languages[1],"variant") != 0))
+			if((v->languages[0] != 0) && (strcmp(&v->languages[1],"variant") != 0) && (memcmp(v->identifier,"mb/",3) != 0))
 			{
 				voices[j++] = v;
 			}
@@ -1593,7 +1699,7 @@ ESPEAK_API const espeak_VOICE **espeak_ListVoices(espeak_VOICE *voice_spec)
 
 ESPEAK_API espeak_VOICE *espeak_GetCurrentVoice(void)
 {//==================================================
-	return(voice_selected);
+	return(&voice_selected);
 }
 
 #pragma GCC visibility pop
