@@ -634,7 +634,7 @@ void wave_init()
 //>
 //<wave_open
 
-void* wave_open(char* the_api)
+void* wave_open(const char* the_api)
 {
   ENTER("wave_open");
   static int once=0;
@@ -1020,9 +1020,9 @@ void *wave_test_get_write_buffer()
 #else
 // notdef USE_PORTAUDIO
 
-void wave_flush(void* theHandler) {}
+
 void wave_init() {}
-void* wave_open(char* the_api) {return (void *)1;}
+void* wave_open(const char* the_api) {return (void *)1;}
 size_t wave_write(void* theHandler, char* theMono16BitsWaveBuffer, size_t theSize) {return theSize;}
 int wave_close(void* theHandler) {return 0;}
 int wave_is_busy(void* theHandler) {return 0;}
